function subPoly = convexDecomposition(site)
    %Delaunay triangulation
    subPoly = delaunay(site);
    
    %Find triangles created by the convex hull
    notIncl = ~ismember(1:size(site,1),convhull(site));
    if isempty(notIncl)
        % Input is convex, just return it
        subPoly=site;
        return;
    end
    reflexVertices = find(notIncl);
    reflexVertices = sort(reflexVertices);
    diffPolygons = {};
    k = 1;
    for i=1:length(reflexVertices)
        if k >= 2 && diffPolygons{k-1}(end) == reflexVertices(i)
            % Merge triangles
            nextV=-1;
            if reflexVertices(i)== size(site,1)
                nextV=1;
            elseif reflexVertices(i) == 1
                nextV=size(site,1);
            else
                nextV=reflexVertices(i)+1;
            end
            diffPolygons{k-1} = [diffPolygons{k-1}, nextV];
            
        else
            if reflexVertices(i) == 1
                diffPolygons{k} = [size(site,1), 1, 2];
            elseif reflexVertices(i) == size(site,1)
                diffPolygons{k} = [size(site,1)-1, size(site,1), 1];
            else
                diffPolygons{k} = [reflexVertices(i)-1, reflexVertices(i), reflexVertices(i)+1];
            end
            k = k + 1;
        end
    end
    
    triangleCell={};
    p=1;
    for t=1:size(subPoly,1)
        inDiffShape=false;
        for k=1:length(diffPolygons)
           if all(ismember(subPoly(t,:),diffPolygons{k}))
              inDiffShape=true; 
              break;
           end
        end
        if not(inDiffShape)
            % If not all corners of triangle are contained in diff
            % shapes
            triangleCell{p}=subPoly(t,:);
            p = p + 1;
        end
    end
    
    % Merge triangles
    % Hertel-Melhorn
    %%{
    subPolys={};
    sp = 1;
    toMerge=1:length(triangleCell);
    while length(toMerge)>=2
        p1=triangleCell{toMerge(1)};
        merged=toMerge(1);
        for j=2:length(toMerge)
           p2=triangleCell{toMerge(j)};
           if sum(ismember(p1,p2)) >= 2
                % Two polys share a diagonal. Need to check if it is
                % essential
                shared1=find(ismember(p1,p2));
                shared2=find(ismember(p2,p1));
                if length(shared1) > 2 || length(shared2) > 2
                    error('Should not be possible to share more than one edge') 
                end
                if shared1(2)-shared1(1) > 1
                    % Wraps around
                    shared1 = fliplr(shared1);
                end
                if shared2(2)-shared2(1) > 1
                    % Wraps around
                    shared2 = fliplr(shared2);
                end
                wrp=@(x,n) (1+mod(x-1,n));
                cross = @(v1,v2) v1(1)*v2(2)-v1(2)*v2(1);
                leftVec1=[p1(shared1(2)),p1(wrp(shared1(2)+1,length(p1)))];
                leftVec2=[p2(shared2(1)),p2(wrp(shared2(1)-1,length(p2)))];
                left=cross(site(leftVec1(2),:)-site(leftVec1(1),:),...
                    site(leftVec2(2),:)-site(leftVec2(1),:)); 

                rightVec1=[p1(shared1(1)),p1(wrp(shared1(1)-1,length(p1)))];
                rightVec2=[p2(shared2(2)),p2(wrp(shared2(2)+1,length(p2)))];
                right=cross(site(rightVec2(2),:)-site(rightVec2(1),:),...
                    site(rightVec1(2),:)-site(rightVec1(1),:)); 

                if left >= 0 && right >= 0
                    % Merged poly is convex, can remove diagonal
                    p1vertices=circshift(p1,length(p1)-shared1(1));
                    p2vertices=circshift(p2,length(p2)-shared2(1)+1);
                    p2vertices=p2vertices(3:end);
                    mPoly=[
                        p1vertices p2vertices
                    ];
                    p1=mPoly;
                    merged = [merged toMerge(j)];
                end 
           end
        end
        subPolys{sp}=p1;
        sp = sp + 1;
        toMerge = setdiff(toMerge,merged);
    end
    if ~isempty(toMerge)
       % Remaining triangles must be added
       for i=1:length(toMerge)
          subPolys{sp}=triangleCell{toMerge(i)};
          sp = sp + 1;
       end
    end
    triangleCell=subPolys;
    %%}    
    
    %{
    %Check all pairs of triangles to see if they can be merged 
    i = 1;
    j = 2;
    while i < length(triangleCell)
        sharedPoints = ismember(triangleCell{i},triangleCell{j});
    	idSharedPoints = triangleCell{i}(sharedPoints==1);
        %Check the triangles/polygons that share 2 corner points
        if sum(sharedPoints) == 2
        	testPoly = triangleCell;
        	testPoly{i}(sharedPoints==1) = [];
        	testPoly{i} = [testPoly{j}, testPoly{i}]; 
            cross = [0 0];
        	for m = 1:length(idSharedPoints)
            	n = mod(m, size(site,1))+1;
            	if idSharedPoints(m) == 1
                	k=size(site,1);
                else 
                	k = idSharedPoints(m)-1;
                end
                vec1 = site(n,:)-site(m,:);
                vec2 = site(k,:)-site(m,:);
                cross(m) = vec1(1)*vec2(2)-vec1(2)*vec2(1); 
            end
            %If both interior angles are convex without the diagonal,
            %remove the diagonal
            if cross(1) >= 0 && cross(2) >= 0
            	triangleCell{i} = testPoly{i};
            	triangleCell(j) = [];
            elseif j < (length(triangleCell)-1)
                j = j+1;
            else
                i = i + 1;
                j = i + 1;
            end
        elseif j < length(triangleCell)
            j = j+1;
        else
            i = i + 1;
            j = i + 1;
        end
    end
    %}
    
    %From corner point index to coordinates
    subPoly = cell(1, length(triangleCell));
    for i = 1:length(triangleCell)
        for j = 1:length(triangleCell{i})
            subPoly{i}(j,:) = site(triangleCell{i}(j),:); 
        end
    end
    
end