function x0=generateInitialValues(strategy, seed, SITE, NUM_BUILDINGS)
    switch strategy
        case Configuration.INIT_RANDOM
            x0=randomInitialValues(seed, SITE, NUM_BUILDINGS);
        case Configuration.INIT_UNIFORM
            x0=uniformInitialValues(seed, SITE, NUM_BUILDINGS);
        case Configuration.INIT_OPENZONES
            x0 = [];
            currentSeed = seed;
            while isempty(x0)
                x0 = openzonesRandom(currentSeed, SITE, NUM_BUILDINGS);
                if isequal(currentSeed, seed)
                    currentSeed = currentSeed + 1000;
                else
                    currentSeed = currentSeed + 1;
                end
            end
        case Configuration.INIT_CIRCLES
            x0=circleInitialValues(seed,SITE,NUM_BUILDINGS);
        case Configuration.INIT_ANGLEDOPENZONES
            x0=siteCornerInitialValues(seed,SITE,NUM_BUILDINGS);
        case Configuration.INIT_SAMEORIENTATION
            x0 = [];
            currentSeed = seed;
            while isempty(x0)
                x0 = sameOrientationRandom(currentSeed, SITE, NUM_BUILDINGS);
                if isequal(currentSeed, seed)
                    currentSeed = currentSeed + 1000;
                else
                    currentSeed = currentSeed + 1;
                end
            end
        otherwise
            error('Invalid strategy for initial value generator')
    end

end

function x0 = siteCornerInitialValues(seed,SITE,NUM_BUILDINGS)
    rng(seed);
    %If it is site 103
    if size(SITE,1) == 17
        SITE = [10, 10; 29.04, 62.03; 60.78, 41.21; 66.91, 50.93; 91.12, 36.58; 85.89, 28.06; 120, 7.33; 100.4, -23.24; 75.55, -7.33; 61.93, -28.61];
    end
    notIncl = ~ismember(1:size(SITE,1),convhull(SITE));
    reflexVertices = find(notIncl);
    reflexVertices = sort(reflexVertices);
    convexVertices = find(~notIncl);
    convexVertices = sort(convexVertices);
    x0 = zeros(NUM_BUILDINGS, 7);
    availableEdges = validSiteEdges(SITE);
    [~, uniqueIdx] = unique(availableEdges);
    index = setdiff( 1:numel(availableEdges), uniqueIdx );
    availableCorners = availableEdges(index);
    availableCorners = unique(availableCorners);
    %availableCorners = 1:size(SITE,1);
    buildingsLeft = NUM_BUILDINGS;
    listOfBuildings = randperm(length(availableCorners));
    n=1;
    while buildingsLeft > 0
        nCornerPoints = length(availableCorners);
        if nCornerPoints == 0
            randomBuilding = listOfBuildings(n);
            x0(NUM_BUILDINGS-buildingsLeft+1,:) = x0(randomBuilding,:);
            w = x0(randomBuilding,3);
            r = x0(randomBuilding,7);
            notRotatedRandomCenter = [x0(randomBuilding,1), x0(randomBuilding,2) - w - w/sin(r/2)];
            V = notRotatedRandomCenter;
            C = x0(randomBuilding,1:2);
            theta = x0(randomBuilding,5);
            %Rotation matrix
            R = [cos(theta) -sin(theta); sin(theta) cos(theta)];
            randomCenter = transpose(R*transpose((V-C)))+ C;
            x0(NUM_BUILDINGS-buildingsLeft+1,1:2) = randomCenter;
            buildingsLeft = buildingsLeft - 1;
            n = n+1;
        else
            cornerPoint = availableCorners(randi(nCornerPoints));
            if cornerPoint == 1
                edge1 = SITE(2,:) - SITE(1,:);
                edge2 = SITE(size(SITE,1),:) - SITE(1,:);
            elseif cornerPoint == size(SITE,1)
                edge1 = SITE(1,:) - SITE(size(SITE,1),:);
                edge2 = SITE(size(SITE,1)-1,:) - SITE(size(SITE,1),:);
            else
                edge1 = SITE(cornerPoint+1,:) - SITE(cornerPoint,:);
                edge2 = SITE(cornerPoint-1,:) - SITE(cornerPoint,:);
            end
            randomLength = Configuration.MIN_LENGTH + rand()*(Configuration.MAX_LENGTH - Configuration.MIN_LENGTH);
            randomWidth = Configuration.MIN_WIDTH + rand()*(Configuration.MAX_WIDTH - Configuration.MIN_WIDTH);
            if sum(reflexVertices==cornerPoint) == 1
                outerAngle = acos(dot(edge1,edge2)/(norm(edge1)*norm(edge2)));
                r = 2*pi-outerAngle;
                c1 = SITE(cornerPoint,:) + edge1*0.5*randomLength/norm(edge1);
                c2 = SITE(cornerPoint,:) + edge2*0.5*randomLength/norm(edge2);
                line = c2-c1;               
                %randomCenter = c1 + 0.5*line + 0.5*randomWidth*[-line(2), line(1)]/norm(line);
                v = [0, -1];
                theta = atan2(v(1), v(2)) - atan2(-line(2), line(1));
                if theta < 0
                    theta = 2*pi + theta;
                end
                %Rotation matrix
                R = [cos(theta) -sin(theta); sin(theta) cos(theta)];
                c_k = SITE(cornerPoint,:);
                notRotatedRandomCenter = [c_k(1), c_k(2)-(randomWidth/sin(r/2))+randomWidth/2];
                V = notRotatedRandomCenter;
                C = c_k;
                randomCenter = transpose(R*transpose((V-C)))+ C;
            elseif sum(convexVertices==cornerPoint)==1
                outerAngle = acos(dot(edge1,edge2)/(norm(edge1)*norm(edge2)));
                r = outerAngle;
                c1 = SITE(cornerPoint,:) + (edge1/norm(edge1))*(0.5*randomLength + (randomWidth/tan(r/2))) + ([edge1(2), -edge1(1)]/norm(edge1))*randomWidth;
                c2 = SITE(cornerPoint,:) + (edge2/norm(edge2))*(0.5*randomLength + (randomWidth/tan(r/2))) + ([-edge2(2), edge2(1)]/norm(edge2))*randomWidth;
                line = c2-c1;
                v = [0, -1];
                theta = atan2(v(1), v(2)) - atan2(-line(2), line(1));
                if theta < 0
                    theta = 2*pi + theta;
                end
                %Rotation matrix
                R = [cos(theta) -sin(theta); sin(theta) cos(theta)];
                c_k = c1 - (edge1/norm(edge1))*0.5*randomLength;
                notRotatedRandomCenter = [c_k(1), c_k(2)+randomWidth/2];
                V = notRotatedRandomCenter;
                C = c_k;
                randomCenter = transpose(R*transpose((V-C)))+ C;
            end
            x0(NUM_BUILDINGS-buildingsLeft+1,:) = [randomCenter, randomWidth, randomLength, theta, 0.5*randomLength, r];
            buildingsLeft = buildingsLeft - 1;
            availableCorners(availableCorners==cornerPoint) = [];
        end
    end
end
            
 

function x=circleInitialValues(seed,SITE,NUM_BUILDINGS)
    x0 = [];
    currentSeed = seed;
    while isempty(x0)
        [c0,~,exitflag] = optimizeCircles(SITE, NUM_BUILDINGS, currentSeed);
        if exitflag == 1 || exitflag == 2
            for i=1:NUM_BUILDINGS
                x = c0(i,1);
                y = c0(i,2);
                r = c0(i,3);
                randW=(Configuration.MAX_WIDTH-Configuration.MIN_WIDTH)*rand + Configuration.MIN_WIDTH;
                l = 2*sqrt(r^2-(1/4)*randW^2);
                randRot = rand*pi;
                x0(i,:) = [x,y,randW,l,randRot];
            end
        else
            currentSeed = currentSeed+1;
        end
    end
    x = zeros(NUM_BUILDINGS,7);
    for i=1:NUM_BUILDINGS
        x(i,:) = [x0(i,:), x0(i,4)/2, pi];
    end
    x
end

function x0=randomInitialValues(seed, SITE, NUM_BUILDINGS)
    rng(seed);
    siteBoundingBox = [min(SITE); max(SITE)];
    x0 = zeros(NUM_BUILDINGS, 7);
    for i=1:NUM_BUILDINGS
       point = (siteBoundingBox(2,:) - siteBoundingBox(1,:)).*rand(1,2) + siteBoundingBox(1,:);
       width = Configuration.MIN_WIDTH + rand(1)*(Configuration.MAX_WIDTH-Configuration.MIN_WIDTH);
       length = Configuration.MIN_LENGTH + rand(1)*(Configuration.MAX_LENGTH-Configuration.MIN_LENGTH);
       rotation= -pi + 2*pi*rand(1);
       kinkLoc = length/2;
       kinkRot = pi;
       x0(i,:) = [point, width, length, rotation, kinkLoc, kinkRot];
    end
end

function x0=uniformInitialValues(seed, SITE, NUM_BUILDINGS)
    rng(seed);
    siteBoundingBox = [min(SITE); max(SITE)];
    values = zeros(NUM_BUILDINGS, 5);
    for i=1:NUM_BUILDINGS
       point = (siteBoundingBox(2,:) - siteBoundingBox(1,:)).*rand(1,2) + siteBoundingBox(1,:);
       values(i,:) = [point, Configuration.MIN_WIDTH, Configuration.MIN_LENGTH, 0];
    end
    
    options = optimoptions(@fmincon,'Algorithm', 'sqp');
    SITE_INEQUALITIES = createLinearShortestDistance(SITE);

    LOWER_BOUNDS = repmat([min(SITE), Configuration.MIN_WIDTH, Configuration.MIN_LENGTH, -4*pi], NUM_BUILDINGS, 1);
    UPPER_BOUNDS = repmat([max(SITE), Configuration.MAX_WIDTH, Configuration.MAX_LENGTH, 4*pi], NUM_BUILDINGS, 1);
    
    % SHOULD_TRANSFORM_VARIABLES = false here
    x0 = ... 
    fmincon(@spreadUniformlyObjFun, values, [], [], [], [], LOWER_BOUNDS, UPPER_BOUNDS, @(x)confun(x,SITE, SITE_INEQUALITIES, false), options);
end

function x0=openzonesOutsiteSiteInitialValues(seed, SITE, NUM_BUILDINGS)
    %[nSiteCornerPoints,~] = size(SITE);
    validEdges = validSiteEdges(SITE);
    nSiteCornerPoints = size(validEdges,1);
    rng(seed);
    a=Configuration.MIN_WIDTH;
    b=Configuration.MAX_WIDTH;
    w=(b-a)*rand + a;
    values = zeros(NUM_BUILDINGS,5);
    normalVector = zeros(NUM_BUILDINGS,2);
    cosTheta = zeros(NUM_BUILDINGS,1);
    for  i = 1:NUM_BUILDINGS
        level = floor(i / nSiteCornerPoints);
        if mod(i,nSiteCornerPoints) == 0
            level = level - 1;
        end
        
        %{
        fromIndex = mod(i,nSiteCornerPoints);
        if mod(i,nSiteCornerPoints) == 0
            fromIndex = nSiteCornerPoints;
        end
        toIndex = mod(i, nSiteCornerPoints) + 1;
        %}
        j = mod(i - 1, nSiteCornerPoints) +  1;
        fromIndex = validEdges(j,1);
        toIndex = validEdges(j,2);
        %[fromIndex, toIndex] = validEdges(j,:);
        siteCornerPoint=SITE(fromIndex,:);
        nextCornerPoint=SITE(toIndex,:);
        %{
        if i == nSiteCornerPoints
            nextCornerPoint=SITE(1,:);
        else
            nextCornerPoint=SITE(i+1,:);
        end
        %}
        xdiff = nextCornerPoint(1)-siteCornerPoint(1);
        ydiff = nextCornerPoint(2)-siteCornerPoint(2);
        l = min(sqrt(xdiff^2+ydiff^2));
        normalVector(i,:) = -1*([-ydiff, xdiff]/l);
        if ydiff >= 0
            cosTheta(i) = xdiff/l;
        else
            cosTheta(i) = -xdiff/l;
        end
        points = [siteCornerPoint(1)+xdiff/2, siteCornerPoint(2)+ydiff/2]+normalVector(i,:)*(w/2);
        translatedInwardsBy = 0;
        l = min(l,Configuration.MAX_LENGTH);
        if level > 0
            translatedInwardsBy = normalVector(i,:)*level*(0.5*w+0.5*Configuration.MIN_WIDTH+Configuration.WIDTH_OPEN_ZONE);
            l = Configuration.MIN_LENGTH;
        end
        values(i,:) = [points + translatedInwardsBy, w, l, acos(cosTheta(i))];
        %values(i,:) = [points, w, min(l,Configuration.MAX_LENGTH), acos(cosTheta(i))];

    end
    %{
    if i <= NUM_BUILDINGS
        rn=zeros(1,NUM_BUILDINGS-nSiteCornerPoints);
        for j = 1:NUM_BUILDINGS-nSiteCornerPoints
            r = randi([1,i]);
            while ismember(r,rn)
                r = randi([1,i]);
            end
            rn(j)=r;
            k = values(r,[1 2]);
            newpoints = k + normalVector(r,:)*(0.5*w+0.5*Configuration.MIN_WIDTH+Configuration.WIDTH_OPEN_ZONE);
            values(j+nSiteCornerPoints,:) = [newpoints, Configuration.MIN_WIDTH, Configuration.MIN_LENGTH, acos(cosTheta(r))];
        end
    end
    %}
    x0=values;
end

function x0=openzonesRandom(seed, SITE, NUM_BUILDINGS)
    rng(seed);
    validEdges = validSiteEdges(SITE);
    availableEdges = validEdges;
    edges = zeros(2,2,size(availableEdges,1));
    e = 0;
    for i=1:size(availableEdges,1)
        fromIndex = availableEdges(i,1);
        toIndex = availableEdges(i,2);
        edges(:,:,i) = [SITE(fromIndex,:); SITE(toIndex,:)]; 
        e = e + 1;
    end
    buildingsLeft = NUM_BUILDINGS;
    x0 = zeros(NUM_BUILDINGS,5);
    while buildingsLeft > 0
        if e <= 0
            x0=[];
            return
        end
        randomRow = randi([1,size(edges,3)]);
        baseEdge = [edges(1,:,randomRow); edges(2,:,randomRow)];
        edgeVector = baseEdge(2,:) - baseEdge(1,:);
        edgeLength = sqrt(edgeVector(1)^2 + edgeVector(2)^2);
        randomLength = rand()*(min(edgeLength,Configuration.MAX_LENGTH) - Configuration.MIN_LENGTH);
        randomLength = (randomLength + Configuration.MIN_LENGTH);
        %t = rand()*(edgeLength-Configuration.MIN_LENGTH);
        randomStart = rand()*(edgeLength - randomLength);
        fromCorner = baseEdge(1,:) + randomStart*edgeVector/edgeLength;
        toCorner = fromCorner + randomLength*edgeVector/edgeLength;

        lineNormalVariables = [toCorner(1), toCorner(2), fromCorner(1), fromCorner(2), Configuration.MAX_WIDTH];
        xn = transformVariables(lineNormalVariables, 'from');
        x0(NUM_BUILDINGS-buildingsLeft+1,:) = xn;
        edges(:,:,randomRow:size(edges,3)-1) = edges(:,:,randomRow+1:size(edges,3));
        edges(:,:,size(edges,3)) = [];
        e = e - 1;
        if pointDistance(baseEdge(1,:),fromCorner) >= Configuration.MIN_LENGTH
            edges(:,:,size(edges,3)+1) = [baseEdge(1,:); fromCorner];
            e = e + 1;
        end
        if pointDistance(toCorner,baseEdge(2,:)) >= Configuration.MIN_LENGTH
            edges(:,:,size(edges,3)+1) = [toCorner; baseEdge(2,:)];
            e = e + 1;
        end

        buildingsLeft = buildingsLeft - 1;
    end
end

function x0=angledOpenzonesRandom(seed, SITE, NUM_BUILDINGS)
    rng(seed);
    %notIncl = ~ismember(1:size(SITE,1),convhull(SITE));
    %convexVertices = find(~notIncl);
    %convexVertices = sort(convexVertices);
    %availableVertices = convexVertices;
    validEdges = validSiteEdges(SITE);
    availableEdges = validEdges;
    edges = zeros(2,2,size(availableEdges,1));
    e = 0;
    for i=1:size(availableEdges,1)
        fromIndex = availableEdges(i,1);
        toIndex = availableEdges(i,2);
        edges(:,:,i) = [SITE(fromIndex,:); SITE(toIndex,:)]; 
        e = e + 1;
    end
    buildingsLeft = NUM_BUILDINGS;
    x0 = zeros(NUM_BUILDINGS,7);
    while buildingsLeft > 0
        if e <= 0
            x0=[];
            return
        end
        randomRow = randi([1,size(edges,3)]);
        baseEdge = [edges(1,:,randomRow); edges(2,:,randomRow)];
        edgeVector = baseEdge(2,:) - baseEdge(1,:);
        edgeLength = sqrt(edgeVector(1)^2 + edgeVector(2)^2);
        randomLength = rand()*(min(edgeLength,Configuration.MAX_LENGTH) - Configuration.MIN_LENGTH);
        randomLength = (randomLength + Configuration.MIN_LENGTH);
        randomWidth = rand()*(Configuration.MAX_WIDTH-Configuration.MIN_WIDTH);
        randomWidth = Configuration.MIN_WIDTH+randomWidth;
        %t = rand()*(edgeLength-Configuration.MIN_LENGTH);
        randomCenter = baseEdge(1,:) + rand()*edgeVector + 0.6464*randomWidth*([edgeVector(2), -edgeVector(1)]/edgeLength);
        randomStart = rand()*(edgeLength - randomLength);
        fromCorner = baseEdge(1,:) + randomStart*edgeVector/edgeLength;
        toCorner = fromCorner + (randomWidth + 0.5*randomLength)*edgeVector/edgeLength;
        v = [0, -1];
        theta = atan2(v(1), v(2)) - atan2(edgeVector(1), edgeVector(2));   
        theta = theta - pi/4;
        x0(NUM_BUILDINGS-buildingsLeft+1,:) = [randomCenter, randomWidth, randomLength, theta, 0.5*randomLength, pi/2];
        edges(:,:,randomRow:size(edges,3)-1) = edges(:,:,randomRow+1:size(edges,3));
        edges(:,:,size(edges,3)) = [];
        e = e - 1;
        if pointDistance(baseEdge(1,:),fromCorner) >= Configuration.MIN_LENGTH
            edges(:,:,size(edges,3)+1) = [baseEdge(1,:); fromCorner];
            e = e + 1;
        end
        if pointDistance(toCorner,baseEdge(2,:)) >= Configuration.MIN_LENGTH
            edges(:,:,size(edges,3)+1) = [toCorner; baseEdge(2,:)];
            e = e + 1;
        end

        buildingsLeft = buildingsLeft - 1;
    end
end

function d=pointDistance(p1,p2)
    d=sqrt((p2(1) - p1(1))^2 + (p2(2) - p1(2))^2);
end

function bestInitialSolution = perpendicular(SITE, NUM_BUILDINGS)
     %siteCenterPoint = mean(SITE);
    %plotSite(SITE);
    validEdges = validSiteEdges(SITE);
    numValidEdges = size(validEdges,1);
    bestInitialArea = 0;
    bestInitialSolution = zeros(NUM_BUILDINGS, 5);
    for e=1:numValidEdges
        fromIndex = validEdges(e,1);
        toIndex = validEdges(e,2);
        edge = [SITE(fromIndex,:); SITE(toIndex,:)];
        edgeVector = edge(2,:) - edge(1,:);
        length = sqrt(edgeVector(1)^2 + edgeVector(2)^2);
        edgeNormal = -1*[-edgeVector(2),edgeVector(1)] / length;
       
        % Find max directional diameter to calculate number of rows of
        % buildings
        maxNormalDistance = 0;
        for i=1:size(SITE,1)
           if i~=fromIndex && i~=toIndex
               r = SITE(fromIndex,:) - SITE(i,:);
               distance = abs(dot(edgeNormal, r));
               maxNormalDistance = max(maxNormalDistance, distance);
           end
        end
        maxNumRows = floor(maxNormalDistance / (Configuration.MIN_WIDTH + (Configuration.LENGTH_OPEN_ZONE)/2));
        minNumRows = floor(maxNormalDistance / (Configuration.MAX_WIDTH + Configuration.LENGTH_OPEN_ZONE));
    
        x0 = zeros(NUM_BUILDINGS, 5);
        width = Configuration.MIN_WIDTH;
        %if NUM_BUILDINGS < minNumRows
        %    width = Configuration.MAX_WIDTH
        buildingsPerRow = NUM_BUILDINGS / maxNumRows
        %buildingsPerRow = (maxNormalDistance / (NUM_BUILDINGS*Configuration.MIN_WIDTH + (NUM_BUILDINGS-1)*Configuration.LENGTH_OPEN_ZONE));
        [lowerSegment, fullUpperSegment, upperSegment] = getBuildingRow(edge, SITE, fromIndex, toIndex, width + Configuration.LENGTH_OPEN_ZONE);
        %line(lowerSegment(:,1), lowerSegment(:,2))
        remainingLowerSegment = lowerSegment;
        remainingUpperSegment = upperSegment;
        segmentWidth = sqrt((lowerSegment(2,1)-lowerSegment(1,1))^2 + (lowerSegment(2,2)-lowerSegment(1,2))^2);
        buildingsThisRow = floor(segmentWidth / (Configuration.MIN_LENGTH + Configuration.WIDTH_OPEN_ZONE/2));
        estimatedRowsLeft = maxNumRows;
        for b=1:NUM_BUILDINGS
            if buildingsThisRow <= 0 || segmentWidth < Configuration.MIN_LENGTH
                [cLowerSegment, cFullUpperSegment, cUpperSegment] = getBuildingRow(fullUpperSegment, SITE, fromIndex, toIndex, width + Configuration.LENGTH_OPEN_ZONE);
                if isequal(cFullUpperSegment, 0) && isequal(cUpperSegment, 0)
                    % No more room for buildings with open zone, possibly
                    % final row
                    [cLowerSegment, cFullUpperSegment, cUpperSegment] = getBuildingRow(fullUpperSegment, SITE, fromIndex, toIndex, width);
                    if isequal(cFullUpperSegment, 0) && isequal(cUpperSegment, 0)
                        break
                    else
                       lowerSegment = cLowerSegment;
                       fullUpperSegment = cFullUpperSegment;
                       upperSegment = cUpperSegment;
                   end
                else
                   lowerSegment = cLowerSegment;
                   fullUpperSegment = cFullUpperSegment;
                   upperSegment = cUpperSegment;
                end
                
                
                %line(lowerSegment(:,1), lowerSegment(:,2))
                remainingLowerSegment = lowerSegment;
                remainingUpperSegment = upperSegment;
                segmentWidth = sqrt((lowerSegment(2,1)-lowerSegment(1,1))^2 + (lowerSegment(2,2)-lowerSegment(1,2))^2);
                buildingsThisRow = floor(segmentWidth / (Configuration.MIN_LENGTH + Configuration.WIDTH_OPEN_ZONE/2));
                estimatedRowsLeft = estimatedRowsLeft - 1;
                buildingsPerRow = (NUM_BUILDINGS - b + 1)/estimatedRowsLeft
                if buildingsThisRow > buildingsPerRow && buildingsThisRow > 1
                    % Do not fill too many buildings on each row
                    %buildingsThisRow = buildingsThisRow - 1;
                end
            end
            if buildingsPerRow > 1
                upperVector = remainingUpperSegment(1,:) - remainingUpperSegment(2,:);
                newTopRight = remainingUpperSegment(2,:) + (1/buildingsThisRow)*upperVector;
                lowerVector = remainingLowerSegment(1,:) - remainingLowerSegment(2,:);
                newBottomRight = remainingLowerSegment(2,:) + (1/buildingsThisRow)*lowerVector;
            else
                % Max building size on this row
                upperVector = remainingUpperSegment(1,:) - remainingUpperSegment(2,:);
                newTopRight = remainingUpperSegment(2,:) + upperVector;
                lowerVector = remainingLowerSegment(1,:) - remainingLowerSegment(2,:);
                newBottomRight = remainingLowerSegment(2,:) + lowerVector;
            end
            %upperMean = mean([remainingUpperSegment(2,:);newTopRight]);
            %lowerMean = mean([remainingLowerSegment(2,:);newBottomRight]);
            %line(remainingLowerSegment(:,1), remainingLowerSegment(:,2))
            %line([remainingLowerSegment(2,1), newBottomRight(1)], [remainingLowerSegment(2,2), newBottomRight(2)])
            lineNormalVariables = [remainingLowerSegment(2,1), remainingLowerSegment(2,2), newBottomRight(1), newBottomRight(2), width];
            variables = transformVariables(lineNormalVariables, 'from');
            %plotBuildings(variables);
            x0(b,:) = variables;
            remainingUpperSegment = [remainingUpperSegment(1,:); newTopRight];
            remainingLowerSegment = [remainingLowerSegment(1,:); newBottomRight];
            segmentWidth = sqrt((remainingLowerSegment(2,1)-remainingLowerSegment(1,1))^2 + (remainingLowerSegment(2,2)-remainingLowerSegment(1,2))^2);
            buildingsThisRow = buildingsThisRow - 1;
        end
        % Save solution if better than previos;
        if ~any(x0(:,3) <= 0)
            % If any of the widths are zero, not all buildings were able to
            % be placed
            if sum(prod(x0(:,3:4),2), 1) > bestInitialArea
                bestInitialSolution = x0;
            end
        end
    end
    %plotSite(SITE);
    %plotBuildings(bestInitialSolution);
end

function ps = validSiteEdges(SITE)
    [nSiteCornerPoints,~] = size(SITE);
    ps = zeros(2,2);
    k = 1;
    for i=1:nSiteCornerPoints
        j = mod(i, nSiteCornerPoints) + 1;
        c=SITE(i,:);
        n=SITE(j,:);
        if sqrt((n(1)-c(1))^2+(n(2)-c(2))^2) > Configuration.MIN_LENGTH
            ps(k,:) = [i, j];
            k = k + 1;
        end
    end
end

function [lowerSegment, fullUpperSegment, upperSegment]=getBuildingRow(edge, SITE, fromIndex, toIndex, width)
        nextLine = getNextRowLine(edge, width);
        nextLineSegment = getNextRowSegment(nextLine, SITE, fromIndex);
        %line(nextLineSegment(:,1), nextLineSegment(:,2));
        if isequal(nextLineSegment,0)
            lowerSegment = edge;
            fullUpperSegment = 0;
            upperSegment = 0;
            return
        end
        edgeVector = edge(2,:) - edge(1,:);
        edgeNormal = -1*[-edgeVector(2),edgeVector(1)];
        topLeft = nextLine(2,:);
        bottomLeft = edge(2,:);
        if cross2D(edgeNormal, nextLineSegment(2,:)-edge(2,:)) < 0
            topLeft = nextLineSegment(2,:);
            % Also move bottom point the same distance
            bottomLeft = edge(2,:) + (topLeft - nextLine(2,:)); 
        end
        topRight = nextLine(1,:);
        bottomRight = edge(1,:);
        if cross2D(edgeNormal, nextLineSegment(1,:)-edge(1,:)) > 0
            topRight = nextLineSegment(1,:);
            % Also move bottom point the same distance
            bottomRight = edge(1,:) + (topRight - nextLine(1,:)); 
        end
        lowerSegment=[bottomRight; bottomLeft];
        fullUpperSegment = nextLineSegment;
        upperSegment=[topRight; topLeft];
        %line(lowerSegment(:,1), lowerSegment(:,2));
end

function seg=getNextRowSegment(nextLine, SITE, baseFrom)
    numOtherEdges = size(SITE,1) - 1;
    numCorners = numOtherEdges + 1;
    leftPoint = 0;
    rightPoint = 0;
    for i=1:numOtherEdges
        fromNext = mod(baseFrom + i - 1, numCorners) + 1;
        toNext = mod(baseFrom + (i+1) - 1, numCorners) + 1;
        ls1 = [SITE(fromNext,:) ; SITE(toNext,:)];
        t=lineSegmentsIntersectionMultiple(ls1, nextLine);
        %line([ls1(1,1), ls1(2,1)], [ls1(1,2), ls1(2,2)])
        if 0 <= t && t < 1
            if leftPoint == 0
                leftPoint = SITE(fromNext,:) + t*(SITE(toNext,:) - SITE(fromNext,:));
            elseif rightPoint == 0
                rightPoint = SITE(fromNext,:) + t*(SITE(toNext,:) - SITE(fromNext,:));
            else
                error('Found multiple intersection points when slicing site for initial values (perpendicular)')
            end
        end
    end
    if isequal(leftPoint,0) && isequal(rightPoint,0)
        seg = 0;
    else
        seg = [rightPoint; leftPoint];
    end
end

function t=lineSegmentsIntersectionMultiple(ls1,ls2)
   p = ls1(1,:);
   r = ls1(2,:)-ls1(1,:);
   
   q = ls2(1,:);
   s = ls2(2,:) - ls2(1,:);
   
   t = cross2D(q-p,s) / cross2D(r, s);
end

function c=cross2D(v1,v2)
    c = v1(1)*v2(2) - v1(2)*v2(1);
end

function nextLine=getNextRowLine(prevLine, w)
    prevVector = prevLine(2,:)-prevLine(1,:);
    normal = -1*[-prevVector(2), prevVector(1)] / sqrt(prevVector(1)^2+prevVector(2)^2);
    nextLine = prevLine + w*normal;
end

function x0=sameOrientationRandom(seed, SITE, NUM_BUILDINGS)
    % First, select a random side and place a building on it, 
    rng(seed);
    
    % All initial solutions
    x0 = zeros(NUM_BUILDINGS,5);
    validEdges = validSiteEdges(SITE);
    randomRow = randi([1,size(validEdges,1)]);
    fromIndex = validEdges(randomRow,1);
    toIndex = validEdges(randomRow,2);
    baseEdge = [SITE(fromIndex,:); SITE(toIndex,:)];
    edgeVector = baseEdge(2,:) - baseEdge(1,:);
    edgeLength = sqrt(edgeVector(1)^2 + edgeVector(2)^2);
    randomLength = rand()*(min(edgeLength,Configuration.MAX_LENGTH) - Configuration.MIN_LENGTH);
    randomLength = (randomLength + Configuration.MIN_LENGTH);
    %t = rand()*(edgeLength-Configuration.MIN_LENGTH);
    randomStart = rand()*(edgeLength - randomLength);
    fromCorner = baseEdge(1,:) + randomStart*edgeVector/edgeLength;
    toCorner = fromCorner + randomLength*edgeVector/edgeLength;
    
    edges = zeros(2,2,1);
    % Number of available edges
    e = 0;
    if pointDistance(baseEdge(1,:), fromCorner) > Configuration.MIN_LENGTH
        e = e + 1; 
        edges(:,:,e) = [baseEdge(1,:); fromCorner]; 
    end
    if pointDistance(toCorner, baseEdge(2,:)) > Configuration.MIN_LENGTH
        e = e + 1; 
        edges(:,:,e) = [toCorner; baseEdge(2,:)]; 
    end
    
    lineNormalVariables = [toCorner(1), toCorner(2), fromCorner(1), fromCorner(2), Configuration.MIN_WIDTH];
    xn = transformVariables(lineNormalVariables, 'from');
    x0(1,:) = xn;
    % Would like to place buildins perpendicular to other buildings. 
    
    % The next edge that all buildings on this 'row' must lie on
    % Used to move buildings inward on the site
    width = Configuration.MIN_WIDTH + Configuration.LENGTH_OPEN_ZONE;
    upperEdge = baseEdge;
    %currentRowLine = 1;
    buildingsLeft = NUM_BUILDINGS - 1;
    while buildingsLeft > 0
        if e <= 0
            % Get next Upper Edge
            [~, cFullUpperSegment, cUpperSegment] = getBuildingRow(upperEdge, SITE, fromIndex, toIndex, width);
            if isequal(cFullUpperSegment, 0) && isequal(cUpperSegment, 0)
                % No more space in site
                %error('No more space on site');
                x0=[];
                return
                %break
            else
               upperEdge = cFullUpperSegment;
            end
             % No more space on this edge, move forward
            if pointDistance(upperEdge(1,:), upperEdge(2,:)) >= Configuration.MIN_LENGTH
                e = e + 1; 
                edges(:,:,e) = [upperEdge(1,:); upperEdge(2,:)]; 
            else
                % This edge too small, should not happen
                x0=[];
                return
            end

        end
        r = randi([1,e]);
        baseEdge = edges(:,:,r);
        edgeVector = baseEdge(2,:) - baseEdge(1,:);
        edgeLength = sqrt(edgeVector(1)^2 + edgeVector(2)^2);
        randomLength = rand()*(min(edgeLength,Configuration.MAX_LENGTH) - Configuration.MIN_LENGTH);
        randomLength = (randomLength + Configuration.MIN_LENGTH);
        %t = rand()*(edgeLength-Configuration.MIN_LENGTH);
        randomStart = rand()*(edgeLength - randomLength);
        fromCorner = baseEdge(1,:) + randomStart*edgeVector/edgeLength;
        toCorner = fromCorner + randomLength*edgeVector/edgeLength;
        
        % Add building 
        lineNormalVariables = [toCorner(1), toCorner(2), fromCorner(1), fromCorner(2), Configuration.MAX_WIDTH];
        xn = transformVariables(lineNormalVariables, 'from');
        x0(NUM_BUILDINGS-buildingsLeft+1,:) = xn;
        
        % Remove edge from list of available edges
        
        edges(:,:,r:size(edges,3)-1) = edges(:,:,r+1:size(edges,3));
        edges(:,:,size(edges,3)) = [];
        e = e - 1;
        if pointDistance(baseEdge(1,:), fromCorner) > Configuration.MIN_LENGTH
            e = e + 1; 
            edges(:,:,e) = [baseEdge(1,:); fromCorner]; 
        end
        if pointDistance(toCorner, baseEdge(2,:)) > Configuration.MIN_LENGTH
            e = e + 1; 
            edges(:,:,e) = [toCorner; baseEdge(2,:)]; 
        end
        
        buildingsLeft = buildingsLeft - 1;
    end
end

function x0 = sameOrientation(SITE, NUM_BUILDINGS)
    values = zeros(NUM_BUILDINGS, 5); 
    [nSiteCornerPoints,~] = size(SITE); %Find the number of corner points
    i=3;
    %rng('shuffle');
    r = randi([1,nSiteCornerPoints]); %Generate a random site corner point
    
    %Find the corner point opposite to r
    if  mod(r+floor((nSiteCornerPoints/2)),nSiteCornerPoints) == 0
        s = nSiteCornerPoints;
    else
    	s = mod(r+floor((nSiteCornerPoints/2)),nSiteCornerPoints);
    end
    siteCornerPoint = SITE(r,:); 
    ssiteCornerPoint = SITE(s,:);
    
    %Find the neighbour site corner point 
    if r == nSiteCornerPoints
    	nextCornerpoint = SITE(1,:);
    else
    	nextCornerpoint = SITE(r+1,:);
    end
    if s == nSiteCornerPoints
        snextCornerPoint = SITE(1,:);
    else
        snextCornerPoint = SITE(s+1,:);
    end
    %Calculate the normal vector of the oppsite edges
    xdiff = nextCornerpoint(1)-siteCornerPoint(1);
    ydiff = nextCornerpoint(2)-siteCornerPoint(2);
    sxdiff = snextCornerPoint(1)-ssiteCornerPoint(1);
    sydiff = snextCornerPoint(2)-ssiteCornerPoint(2);
    dist = sqrt((nextCornerpoint(1)-ssiteCornerPoint(1))^2+...
        (nextCornerpoint(2)-ssiteCornerPoint(2))^2);
    l=sqrt(xdiff^2+ydiff^2);
    sl=sqrt(sxdiff^2+sydiff^2);
    normalVector = [-ydiff, xdiff]/l;
    snormalVector = [-sydiff, sxdiff]/sl;
    
    %Find the rotation of the buildings
    if ydiff >= 0
        cosTheta = xdiff/l;
    else
        cosTheta = -xdiff/l;
    end
    if sydiff >= 0
        scosTheta = sxdiff/sl;
    else
        scosTheta = -sxdiff/sl;
    end
    %Find the centre point of the two first buildings to be placed opposite
    %of each other parallell to the respective edges
    point = [siteCornerPoint(1)+xdiff/2, siteCornerPoint(2)+ydiff/2]+...
        normalVector*(Configuration.MIN_WIDTH/2);
    values(1,:)=[point, Configuration.MIN_WIDTH,...
        min(l,Configuration.MAX_LENGTH), acos(cosTheta)];
    s_point = [ssiteCornerPoint(1)+sxdiff/2, ssiteCornerPoint(2)+sydiff/2]...
        +snormalVector*(Configuration.MIN_WIDTH/2);
    values(2,:)=[s_point, Configuration.MIN_WIDTH, min(sl,Configuration.MAX_LENGTH), acos(scosTheta)];
    %As long as buildings can be placed from opposite sides without
    %overlap
    while dist >= i*(Configuration.MIN_WIDTH+Configuration.WIDTH_OPEN_ZONE)...
            && i<=NUM_BUILDINGS
        if mod(i,2)~=0
            point(i,:)=point(i-2,:)+normalVector*...
                (Configuration.MIN_WIDTH+Configuration.WIDTH_OPEN_ZONE);
            values(i,:)=[point(i,:), Configuration.MIN_WIDTH,...
                min(l,Configuration.MAX_LENGTH), acos(cosTheta)];
        elseif i==4
            s_point(i,:)=s_point(i-3,:)+snormalVector*...
                (Configuration.MIN_WIDTH+Configuration.WIDTH_OPEN_ZONE);
            values(i,:)=[s_point(i,:), Configuration.MIN_WIDTH, ...
                min(sl,Configuration.MAX_LENGTH), acos(scosTheta)];
        else
            s_point(i,:)=s_point(i-2,:)+snormalVector*...
                (Configuration.MIN_WIDTH+Configuration.WIDTH_OPEN_ZONE);
        	values(i,:)=[s_point(i,:), Configuration.MIN_WIDTH, ...
            	min(sl,Configuration.MAX_LENGTH), acos(scosTheta)];
        end
        i=i+1;
    end
    f=i;
    %If no more buildings can be placed without overlap they are placed
    %normal to the ones already in place
    if f <= NUM_BUILDINGS
        for j = f:NUM_BUILDINGS
        newNormalVector=[xdiff,ydiff]/l;
        %The first building is placed
        if j==f
            point(j,:)=values((j-1),[1,2]) +(newNormalVector)*(2*Configuration.LENGTH_OPEN_ZONE+min(Configuration.MAX_LENGTH/2,l/2));
            values(j,:)=[point(j,1), point(j,2),Configuration.MIN_WIDTH,...
                Configuration.MIN_LENGTH, acos(cosTheta)];
        else
            point(j,:)=values((j-1),[1,2]) + normalVector*(Configuration.LENGTH_OPEN_ZONE+Configuration.MIN_LENGTH);
            values(j,:)=[point(j,1), point(j,2),Configuration.MIN_WIDTH,...
            Configuration.MIN_LENGTH, acos(cosTheta)];
        end
        end
    end
    x0=values;
end

