function staticObj = getStaticObjects(site)
    %Assumes a non-convex site as input

    %Create the convex hull
    convHull = convhull(site);
    CornerPoints = 1:size(site,1);
    lastCornerPoint = CornerPoints(length(CornerPoints));
    listReflexVertices = ~ismember(CornerPoints,convHull);
    newRow = 1:size(site,1);
    reflexVertices = [listReflexVertices; newRow];
    
    %Determine which corner points are not included in the convex hull
    notIncluded = reflexVertices(2,reflexVertices(1,:)==1);
    grouped = mat2cell(notIncluded, 1, diff([0, find(diff(notIncluded) ~= 1), length(notIncluded)]));
    numStaticObj = length(grouped);
    
    if (grouped{1}(1) == 1) && (grouped{numStaticObj}(length(grouped{numStaticObj})) == lastCornerPoint)
        grouped{1} = [grouped{numStaticObj} grouped{1}];
        grouped(numStaticObj) = [];
    end
    
    %Create a cell array where each cell represets a static object
    %represented by the corner point number
    staticCorners = cell(1,numStaticObj);
    for i = 1:length(grouped)
        consecutiveVertices = grouped{i};
        if consecutiveVertices(1) == 1
            firstCorner = lastCornerPoint;
        else
            firstCorner = consecutiveVertices(1) - 1;
        end
        if consecutiveVertices(length(consecutiveVertices)) == lastCornerPoint
            lastCorner = 1;
        else
            lastCorner = consecutiveVertices(length(consecutiveVertices))+1;
        end
        staticCorners{i} = [firstCorner, grouped{i}, lastCorner];
    end
    
    %From corner point index to coordinates
    staticObj = cell(1, numStaticObj);
    for i = 1:numStaticObj
        for j = 1:length(staticCorners{i})
        staticObj{i}(j,:) = site(staticCorners{i}(j),:); 
        end
    end
    
end