function cornerPoints = kinkedAltVariables(b, varargin)
    x1 = b(1);
    y1 = b(2);
    x2 = b(3);
    y2 = b(4);
    w = b(5);
    k = b(6);
    h = b(7);
    
    p1 = [x1 y1];
    p2 = [x2 y2];
    p1top2 = p2-p1; 
    l = norm(p1top2);
    theta = atan((y2-y1)/(x2-x1));      %Building rotation
    alpha = atan(h/k);                  %Angle of p1 corner point
    beta = atan(h/(1-k));                 %Angle of p2 corner point
    r = pi - alpha - beta;              %Kink angle
    kn = p1 + k*p1top2;
    c_k = kn + h*[-p1top2(2), p1top2(1)];  %Kink corner point
    vecp1 = [p1(1)-c_k(1), p1(2)-c_k(2)];
    vecp2 = [p2(1)-c_k(1), p2(2)-c_k(2)];
    lengthVecp1 = norm(vecp1);
    lengthVecp2 = norm(vecp2);
    
    % If bufferzone is added
    extraWidth = 0;
    extraLengthp1 = [0 0];
    extraLengthp2 = [0 0];
    extraKink = 0;
    extraHeight = 0;
    if length(varargin) == 2
        if varargin{1} == true
            extraLengthp1 = Configuration.WIDTH_OPEN_ZONE*(vecp1+[-vecp1(2), vecp1(1)])/lengthVecp1;
            extraLengthp2 = Configuration.WIDTH_OPEN_ZONE*(vecp2+[vecp2(2), -vecp2(1)])/lengthVecp2;
            extraWidth = 2*Configuration.WIDTH_OPEN_ZONE;
            extraHeight = norm(extraLengthp1)*sin(pi/4+alpha)-Configuration.WIDTH_OPEN_ZONE/sin(r/2);
        end
        if varargin{2} == true
            extraLengthp1 = Configuration.LENGTH_OPEN_ZONE*[-vecp1(2), vecp1(1)]/lengthVecp1;
            extraLengthp2 = Configuration.LENGTH_OPEN_ZONE*[vecp2(2), vecp2(1)]/lengthVecp2;
            extraWidth = 2*Configuration.LENGTH_OPEN_ZONE;
            %extraKink = -cos(pi/2-alpha)*Configuration.LENGTH_OPEN_ZONE;
            extraHeight = sin(pi/2-alpha)*Configuration.LENGTH_OPEN_ZONE-Configuration.LENGTH_OPEN_ZONE/sin(r/2);
        end
    end
    P1 = p1 + extraLengthp1;
    P2 = p2 + extraLengthp2;
    P1TOP2 = P2-P1;
    W = w + extraWidth;
    K = k*l + extraKink;
    H = h*l + extraHeight;
    KN = P1 + K*P1TOP2/norm(P1TOP2);
    extraKinkLength = W/tan(r/2); 
    
    %Calculating corner points coordinates
    c1 = P1 + W*([vecp1(2), -vecp1(1)]/lengthVecp1);
    c_k = KN + H*[-p1top2(2), p1top2(1)]/l;
    c2 = c1 + (-vecp1)*(1+(extraKinkLength/lengthVecp1));
    c3 = P2 + W*[-vecp2(2), vecp2(1)]/lengthVecp2;
    
    cornerPoints = [P1; c_k; P2; c3; c2; c1];
end
    