function [xout,fval, history]= masterOptimize(SITE,ZONES,ZONES_HEIGHTS,MEAN_HEIGHT,NUM_BUILDINGS, MODE, INITIAL_SEED, DIST_ITERS)
    xout=[];
    fval=0;
    switch MODE
        case 'simple'
            heights=[
                min(ZONES_HEIGHTS);
                %max(ZONES_HEIGHTS);
                %MEAN_HEIGHT
            ];
            currentBest=[];
            currentBestFval=Inf;
            for h=1:length(heights)
                height=heights(h);
                %bestX=[];
                %bestF=Inf;
                curSeed=INITIAL_SEED;
                for it=1:DIST_ITERS
                    [xout, ~, exitflag, ~]=optimize(SITE, NUM_BUILDINGS, curSeed, Configuration.CENTER, Configuration.INIT_RANDOM, height);
                    if exitflag==1 || exitflag==2
                       % Optimize heights
                       btzh=buildingsToZoneHeight(xout,ZONES,ZONES_HEIGHTS);
                       stories=optimizeStories(xout, MEAN_HEIGHT,btzh);
                       temp=xout;
                       temp(:,8)=stories;
                       fout=objfun3d(temp,Configuration.CENTER);
                       if fout < currentBestFval
                          currentBest=temp;
                          currentBestFval=fout;
                       end
                    end
                    curSeed=curSeed + 1;
                end
            end
            xout=currentBest;
            fval=currentBestFval;
            
        case 'uniform'
            % Merge zones
            [mergedZones,mergedZonesHeights,zToMZ]=mergeZones(ZONES,ZONES_HEIGHTS);
            btoiz=distributeUniformly(ZONES,NUM_BUILDINGS);
            btoz=zeros(NUM_BUILDINGS,1);
            for i=1:NUM_BUILDINGS
                btoz(i)=zToMZ(btoiz(i));
            end
            fixedBuildings=[];
            infeasible=true;
            xout=[];
            curSeed=INITIAL_SEED;
            while infeasible
                [xsubtemp,~,exitflag,~]=subOptimize(SITE, mergedZones, mergedZonesHeights, btoz, ZONES, btoiz, NUM_BUILDINGS,fixedBuildings, MEAN_HEIGHT, curSeed, Configuration.CENTER);
                %[xsubtemp,~,exitflag,~]=subOptimize(SITE, ZONES, ZONES_HEIGHTS, btoiz, ZONES, btoiz, NUM_BUILDINGS,fixedBuildings, MEAN_HEIGHT, curSeed, Configuration.CENTER);
                if exitflag==1 || exitflag==2
                   infeasible=false; 
                   xout=xsubtemp;
                else
                    curSeed=curSeed + 1;
                end
            end
            % Round heights
            xout(:,end)=roundHeights(buildingAreas(xout),xout(:,end),btoiz,ZONES_HEIGHTS,MEAN_HEIGHT);
            fval=-1*objfun3d(xout,Configuration.CENTER);
        case 'volume'
            maxBuildingsInZones=ones(length(ZONES),1)*NUM_BUILDINGS;
            disp('Initial distributoin over zones:');
            btoiz=distributeVolumeGreedy(ZONES,ZONES_HEIGHTS,MEAN_HEIGHT,NUM_BUILDINGS,maxBuildingsInZones)
            disp('-----');
            btoz=zeros(NUM_BUILDINGS,1);
            %Merge zones
            [mergedZones,mergedZonesHeights,zToMZ]=mergeZones(ZONES,ZONES_HEIGHTS);
            for i=1:NUM_BUILDINGS
                btoz(i)=zToMZ(btoiz(i));
            end
            fixedBuildings=[];
            optimal=false;
            xout=[];
            curSeed=INITIAL_SEED;
            currentBest=[];
            currentBestFval=Inf;
            visitedInitialDistribution=[btoiz'];
            history=[];
            while ~optimal
                bestX=[];
                bestF=Inf;
                for i=1:DIST_ITERS
                    tic;
                    [xsubtemp,fval,exitflag,~]=subOptimize(SITE, mergedZones, mergedZonesHeights, btoz, ZONES, btoiz, NUM_BUILDINGS,fixedBuildings, MEAN_HEIGHT, curSeed, Configuration.CENTER);
                    xtrans=xsubtemp';
                    xtrans=xtrans(:);
                    history=[history;
                        fval, exitflag, toc, btoiz',xtrans'
                    ];
                    %[xsubtemp,~,exitflag,~]=subOptimize(SITE, ZONES, ZONES_HEIGHTS, btoiz, ZONES, btoiz, NUM_BUILDINGS,fixedBuildings, MEAN_HEIGHT, curSeed, Configuration.CENTER);
                    if exitflag==1 || exitflag==2
                       %infeasible=false; 
                       xout=xsubtemp;
                       % Check status of solution
                       if fval<bestF
                           disp('Updating best...:');
                           bestF=fval;
                           bestX=xout
                           disp('------');
                       end
                    end
                    curSeed=curSeed + 1
                end
                disp('Iterations on distribution done...');
                if isempty(bestX) && isempty(currentBest)
                    disp('No feasible solutions');
                    % Too ambitious distribution of buildings in zones, set
                    % constraint on that with least area pr building (ie
                    % the densest)
                    [~,minI]=min(arrayfun(@(zi) polygonArea(ZONES{zi}/sum(btoiz==zi)), 1:length(ZONES)));
                    maxBuildingsInZones(minI) = sum(btoiz==minI) - 1;
                    disp('New distribution...:');
                    btoiz=distributeVolumeGreedy(ZONES,ZONES_HEIGHTS,MEAN_HEIGHT,NUM_BUILDINGS,maxBuildingsInZones)
                    if isempty(btoiz)
                       % No more room
                       disp('No more room for new distribution. Terminate with best');
                       break;
                    end
                    disp('---------');
                    if ismember(btoiz',visitedInitialDistribution,'rows')
                        % Cycling, abort
                        disp('Distribution previously tried. Terminating');
                        break;
                    else
                        visitedInitialDistribution=[visitedInitialDistribution; btoiz'];
                    end
                    for i=1:NUM_BUILDINGS
                        btoz(i)=zToMZ(btoiz(i));
                    end
                    continue
                elseif isempty(currentBest) || bestF < currentBestFval
                    disp('Best from distribution is better than overall best...');    
                    currentBest=bestX;
                    currentBestFval=bestF;
                    disp('----------');
                else
                    % Did not improve on solution that produced currently
                    % best solution, terminate
                    optimal=true;
                    break;
                end
                % If no feasible solutions found, redo distribution
                % DO HERE
                
                % Else, see if one can do any profitable swaps
                [~,hid]=sort(ZONES_HEIGHTS,'descend');
                % Check if there is volume to gain by moving buildings into
                % lower zones
                MIN_AREA=Configuration.MIN_LENGTH*Configuration.MIN_WIDTH+Configuration.MIN_WIDTH^2;
                hasSwapped=false;
                for i=1:length(hid)-1
                    zid=hid(i);
                    if sum(btoiz==zid)==0
                       % No more buildings in this zone
                       continue
                    end
                    %bids=find(btoiz==zid);
                    Bi=bestX(btoiz==zid,:);
                    %remainingVolI=(polygonArea(ZONES{zid})-sum(buildingAreas(Bi)))*ZONES_HEIGHTS(zid); % - (-objfun3d(Bi,Configuration.CENTER));
                    vols=buildingAreas(Bi).*Bi(:,end);
                    smallestBuildingVolI=min(vols);
                    for j=i+1:length(hid)
                         zjd=hid(j);
                         Bj=bestX(btoiz==zjd,:);
                         zjArea=polygonArea(ZONES{zjd});
                         remainingAreaJ=zjArea - sum(buildingAreas(Bj));
                         remainingVolJ=zjArea*ZONES_HEIGHTS(zjd);% - (-objfun3d(Bj,Configuration.CENTER));
                         if (remainingVolJ > smallestBuildingVolI) && (remainingAreaJ > MIN_AREA)  
                            bids=find(btoiz==zid);
                            % Swap one building to this zone j, as it has
                            % more volume remaining
                            temp=btoiz;
                            btoiz(bids(1))=zjd
                            if ismember(btoiz',visitedInitialDistribution,'rows')
                                % Cycling, abort
                                disp('Distribution previously tried. Terminating');
                                break;
                            else
                                visitedInitialDistribution=[visitedInitialDistribution; btoiz'];
                            end
                            for k=1:NUM_BUILDINGS
                                btoz(k)=zToMZ(btoiz(k));
                            end
                            disp('Swap found, re-optimizing....');
                            disp('Old building-to-zones...:')
                            temp
                            disp('New building-to-zones...:')
                            btoiz
                            hasSwapped=true;
                            break;
                         end
                    end
                    if hasSwapped
                       break; 
                    end
                end
                % If NO SWAPS - optimal solution found
                if ~hasSwapped
                   disp('No possible swaps.. Solution optimal');
                   optimal=true; 
                end
            end
            % Round heights
            %xout(:,end)=roundHeights(buildingAreas(xout),xout(:,end),btoiz,ZONES_HEIGHTS,MEAN_HEIGHT);
            % Round down
            %tmp=currentBest;
            %tmp(:,8)=floor(tmp(:,8));
            %xout=tmp;
            xout=currentBest;
            fval=objfun3d(xout,Configuration.CENTER);
    end

    % Plot solution
end


function btzh=buildingsToZoneHeight(x,ZONES,ZONES_HEIGHTS)
    btzh=zeros(size(x,1),1);
    for i=1:size(x,1)
        bCorners=altCornerPoints(x(i,:),Configuration.CENTER);
        zminh=Inf;
        for j=1:length(ZONES)
            if sum(any(siteOverlapNonConvex(bCorners,ZONES{j}) < 0)) >= 1
               % Building overlaps with zone
               if ZONES_HEIGHTS(j) < zminh
                  zminh=ZONES_HEIGHTS(j);
               end
            end
        end
        if zminh == Inf
            error('Building does not overlap with any zones.'); 
        end
        btzh(i)=zminh;
    end
end


function btoz=distributeUniformly(ZONES,NUM_BUILDINGS)
    minMaxInZones=zeros(length(ZONES),2);
    for i=1:length(ZONES)
       minMaxInZones(i,:)=minMaxNumberOfBuildings(polygonArea(ZONES{i})); 
    end
    if sum(minMaxInZones(:,2)) < NUM_BUILDINGS
       error('Too many buildings to initiate uniformly'); 
    end
    [~,idx]=sortrows(minMaxInZones,[-1 -2]);
    %buildingsInZones=zeros(length(ZONES),1);
    btoz=zeros(NUM_BUILDINGS,1);
    nb=1;
    cur=1;
    while nb<=NUM_BUILDINGS
        % Check how many buildings are placed in same (current) zone
        if sum(any(btoz == idx(cur)) < minMaxInZones(idx(cur),2))
           btoz(nb) = idx(cur);
           nb = nb + 1;
        end
        cur = cur + 1;
        if cur > length(ZONES)
            cur = 1;
        end
    end
end


function btoz=distributeVolumeGreedy(ZONES,ZONES_HEIGHTS,MEAN_HEIGHT,NUM_BUILDINGS,maxBuildingsInZones)
    % Successively place buildings in zones (or remainders of
    % zones) that provide the best gain in volume
    [~,hid]=sort(ZONES_HEIGHTS,'descend');
    remainingAreas=zeros(length(ZONES),1);
    for i=1:length(ZONES)
       remainingAreas(i) = 0.5*polygonArea(ZONES{i}); 
    end
    %remainingVolumes=remainingAreas.*ZONES_HEIGHTS;
    placedZones=zeros(length(ZONES),1);
    notFullZones=1:length(ZONES);
    % A nx3 matrix where col1 is where build    ing n has been placed, and col
    % 2 is its area, and 3 its height. Initialize all buildings with min
    % heights
    placedBuildings=zeros(NUM_BUILDINGS,3);
    placedBuildings(:,3)=ones(NUM_BUILDINGS,1)*Configuration.MIN_HEIGHT;
    %placedBuildings(:,2:3)=ones(NUM_BUILDINGS,2)*Configuration.MIN_HEIGHT;
    numPlaced=0;
    noMoreRoom=false;
    while numPlaced < NUM_BUILDINGS
        if isempty(notFullZones)
           noMoreRoom=true;
           break 
        end
        % Calculate max remaining height before mean height constraint
        % becomes binding
        remainingAllowedHeight=NUM_BUILDINGS*MEAN_HEIGHT-sum(placedBuildings(:,3));
        fun=@(z)placeableVolume(remainingAreas(z),ZONES_HEIGHTS(z),...
            remainingAllowedHeight);
        placeableVolumes=arrayfun(fun,notFullZones);
        [~,maxI]=max(placeableVolumes);
        maxI=notFullZones(maxI);
        placedZones(maxI)=placedZones(maxI)+1;
        numPlaced=numPlaced+1;
        placedBuildings(numPlaced,1)=maxI;
        % Update remain ing areas
        [~,a,h]=placeableVolume(remainingAreas(maxI), ZONES_HEIGHTS(maxI), remainingAllowedHeight);
        placedBuildings(numPlaced,2)=a;
        placedBuildings(numPlaced,3)=h;
        remainingAreas(maxI)=remainingAreas(maxI) - a;
        if placedZones(maxI) >= maxBuildingsInZones(maxI)
            notFullZones = setdiff(notFullZones,maxI);
        end
    end
    if noMoreRoom
        btoz=[];
        return
    end
    % Greed is not optimal,
    % adjust according to footprint
    %canSwap=true;
    %while canSwap
    %     remainingAllowedHeight=NUM_BUILDINGS*MEAN_HEIGHT-sum(placedBuildings(:,3));
    %     [~,rid]=sort(remainingAreas);
    %     for i=1:length(ZONES)
    %         for j=1:lengh(ZONES)
    %             
    %         end
    %     end
    %end
    
    btoz=placedBuildings(:,1);
end

function [v,a,h]=placeableVolume(remainingArea, heightZone, remainingAllowedHeight)
    maxBuildingArea=Configuration.MAX_LENGTH*Configuration.MAX_WIDTH + ...
        Configuration.MAX_WIDTH^2;
    a=min(remainingArea,maxBuildingArea);
    h=min(remainingAllowedHeight,heightZone);
    v=a*h;
end

function rounded=roundHeights(areas,heights,btoz,maxHeights,meanHeight)
    % First floor all real heights, then round up descending by floor area
    % if possible
    N=length(areas);
    [~,idx]=sortrows(areas,[-1]);
    rounded=floor(heights);
    for i=1:N
       tmp = rounded;
       bi=idx(i);
       zi=btoz(bi);
       % While we havent reached max stories in zone or mean height, keep
       % increasing height
       while tmp(bi) < maxHeights(zi) && sum(tmp) < N*meanHeight
          tmp(bi)=tmp(bi)+1;
          rounded=tmp; 
       end
    end
end


function areas=buildingAreas(x)
    areas=zeros(size(x,1),1);
    for i=1:size(x,1)
       areas(i)=-1*objfun(x(i,1:end-1),Configuration.CENTER); 
    end
end