function cornerPoints = midLineCornerPoints(b, varargin)
    x1 = b(1);
    y1 = b(2);
    x2 = b(3);
    y2 = b(4);
    w = b(5);
    k = b(6);
    h = b(7);
    
    p1 = [x1 y1];
    p2 = [x2 y2];
    p1top2 = p2-p1; 
    kn = p1 + k*p1top2;
    c_kin = kn + [-p1top2(2), p1top2(1)]*h;
    l = norm(p1top2);
    vecp1 = p1 - c_kin;
    vecp2 = p2 - c_kin;
    lengthVecp1 = norm(vecp1);
    lengthVecp2 = norm(vecp2);
    alpha = atan(h/k);              %Angle of point p1
    beta = atan(h/(1-k));           %Angle of point p2
    r = pi-beta-alpha;              %Kink angle
    theta = atan((y2-y1)/(x2-x1));  %Building rotation
    
    % If bufferzone is added
    extraWidth = 0;
    extraLengthp1 = [0 0];
    extraLengthp2 = [0 0];
    extraKink = 0;
    extraHeight = 0;
    if length(varargin) == 2
        if varargin{1} == true
            extraLengthp1 = Configuration.WIDTH_OPEN_ZONE*vecp1/norm(vecp1);
            extraLengthp2 = Configuration.WIDTH_OPEN_ZONE*vecp2/norm(vecp2);
            extraWidth = 2*Configuration.WIDTH_OPEN_ZONE;
            extraKink = norm(extraLengthp1)*cos(theta+alpha);
            %extraKink = Configuration.WIDTH_OPEN_ZONE*(1-1/tan(r/2));
            extraHeight = norm(extraLengthp1)*sin(theta+alpha);
        end
        if varargin{2} == true
            extraWidth = 2*Configuration.LENGTH_OPEN_ZONE;
        end
    end
    P1 = p1 + extraLengthp1;
    P2 = p2 + extraLengthp2;
    L = norm(P2-P1);
    W = w + extraWidth;
    K = k*l + extraKink;
    H = h*l + extraHeight;
    addedKinkLength = (W/2)/tan(r/2);
    a = sqrt(K^2 + H^2);
    
    c1 = P1 + (W/2)*[-vecp1(2), vecp1(1)]/lengthVecp1;
    c2 = P1 + (W/2)*[vecp1(2), -vecp1(1)]/lengthVecp1;
    c_k = c1 - (a-addedKinkLength)*vecp1/lengthVecp1;
    c3 = c2 - (a+addedKinkLength)*vecp1/lengthVecp1;
    c4 = P2 + (W/2)*[-vecp2(2), vecp2(1)]/lengthVecp2;
    c5 = P2 + (W/2)*[vecp2(2), -vecp2(1)]/lengthVecp2;
    
    cornerPoints = [c1; c_k; c5; c4; c3; c2];
end