function cornerPoints = midVectorCornerPoints(b, varargin)
    x = b(1);
    y = b(2);
    vx1 = b(3);
    vy1 = b(4);
    vx2 = b(5);
    vy2 = b(6);
    w = b(7);

    p1 = [x, y];
    vec1 = [vx1, vy1];
    vec2 = [vx2, vy2];
    p2 = p1 + (vec1 + vec2);
    L1 = norm(vec1);
    L2 = norm(vec2);

    normalVec1 = ([vec1(2), -vec1(1)])/L1;
    normalVec2 = ([vec2(2), -vec2(1)])/L2;

    extraP1 = 0;
    extraWidth = 0;
    extraP2 = 0;
    if length(varargin) == 2
        if varargin{1} == true
        extraP1 = (-vec1/L1)*Configuration.WIDTH_OPEN_ZONE;
        extraWidth = 2*Configuration.WIDTH_OPEN_ZONE;
        extraP2 = (vec2/L2)*Configuration.WIDTH_OPEN_ZONE;
        end
    end
    
    P1 = p1 + extraP1;
    P2 = p2 + extraP2;
    W = w + extraWidth;
    
    c1 = P1 + (W/2)*normalVec1;
    c5 = P1 - (W/2)*normalVec1;
    c2 = P2 + (W/2)*normalVec2;
    c3 = P2 - (W/2)*normalVec2;
    
    den = vec1(1)*vec2(2)-vec1(2)*vec2(1); 
    if den == 0
        ck = c1 + vec1;
        c4 = c5 + vec1;
    else
        u = rayIntersection(c1, vec1, c2, -vec2);
        ck = c1 + u*vec1;
        t = rayIntersection(c5, vec1, c3, -vec2);
        c4 = c5 + t*vec1;
    end
    cornerPoints =  [c1; ck; c2; c3; c4; c5];

end

function [u,v]=rayIntersection(as,ad,bs,bd)
    % p = as + ad * u
    % p = bs + bd * v
    % u := (as.y*bd.x + bd.y*bs.x - bs.y*bd.x - bd.y*as.x ) / (ad.x*bd.y - ad.y*bd.x)
    u = (as(2)*bd(1) + bd(2)*bs(1) - bs(2)*bd(1) - bd(2)*as(1) ) / (ad(1)*bd(2) - ad(2)*bd(1));
    %v := (as.x + ad.x * u - bs.x) / bd.x
    v = (as(1) + ad(1) * u - bs(1)) / bd(1);

end