function [f,g] = objfun(x,SHOULD_TRANSFORM_VARIABLES)

    if SHOULD_TRANSFORM_VARIABLES == Configuration.LINEWIDTH
        f = -totalAreaLineWidth(x);
    elseif SHOULD_TRANSFORM_VARIABLES == Configuration.LOWERLINEWIDTH
        f = -totalAreaLowerLineWidth(x);
    elseif SHOULD_TRANSFORM_VARIABLES == Configuration.VECTOR || ...
            SHOULD_TRANSFORM_VARIABLES == Configuration.UPDOWNVECTOR
        f = -totalAreaVector(x);
    elseif SHOULD_TRANSFORM_VARIABLES == Configuration.MIDVECTOR
        f = -totalAreaMidVector(x);
    else
        f = -totalArea(x);
    end
    
    % Jacobian
    g = zeros(size(x,1),size(x,2));
    for i=1:size(x,1)
        if SHOULD_TRANSFORM_VARIABLES == Configuration.LINEWIDTH
            x1 = x(i,1);
            y1 = x(i,2);
            x2 = x(i,3);
            y2 = x(i,4);
            w = x(i,5);
            k = x(i,6);
            h = x(i,7);
            l = sqrt((x2-x1)^2 + (y2-y1)^2); %Length of line between points
            ex = sqrt(h^2+k^2)+sqrt(h^2+(1-k)^2); %length of kinked midpoint-line
            dx1 = -w*(x2-x1)*ex/l;
            dy1 = -w*(y2-y1)*ex/l;
            dx2 = w*(x2-x1)*ex/l;
            dy2 = w*(y2-y1)*ex/l;
            dw = l*ex;
            dk = w*l*((k/sqrt(h^2+k^2))-((1-k)/sqrt(h^2+(1-k)^2)));
            dh = w*l*((h/sqrt(h^2+k^2))+(h/sqrt(h^2+(1-k)^2)));
            g(i,:) = (-1)*[dx1, dy1, dx2, dy2, dw, dk, dh];
        elseif SHOULD_TRANSFORM_VARIABLES == Configuration.LOWERLINEWIDTH
            x1 = x(i,1);
            y1 = x(i,2);
            x2 = x(i,3);
            y2 = x(i,4);
            w = x(i,5);
            k = x(i,6);
            h = x(i,7);
            l = sqrt((x2-x1)^2 + (y2-y1)^2); %Length of line between points
            r = pi - atan(h/k) - atan(h/(1-k));
            ex = sqrt(h^2+k^2)+sqrt(h^2+(1-k)^2); %Length
            dx1 = -w*(x2-x1)*ex/l;
            dy1 = -w*(y2-y1)*ex/l;
            dx2 = w*(x2-x1)*ex/l;
            dy2 = w*(y2-y1)*ex/l;
            dw = l*ex + 2*w/tan(r/2);
            dk = l*w*((k/sqrt(h^2+k^2)) - ((1-k)/sqrt(h^2+(1-k)^2))) - 0.5*w^2*((h/(k^2*((h^2/k^2)+1))) - (h/((1-k)^2*((h^2/(1-k)^2)+1))))*(csc(0.5*(-atan(h/(1-k)))-atan(h/k)+pi))^2;
            dh = l*w*((h/sqrt(h^2+k^2)) + (h/sqrt(h^2+(1-k)^2))) - 0.5*w^2*((-1/(k*((h^2/k^2)+1))) -(1/((1-k)*((h^2/(1-k)^2)+1)))) * (csc(0.5*(-atan(h/(1-k))-atan(h/k)+pi)))^2;
            g(i,:) = [-dx1, -dy1, -dx2, -dy2, -dw, -dk, -dh];
        elseif SHOULD_TRANSFORM_VARIABLES == Configuration.MIDVECTOR
            x1=x(i,1);
            y1=x(i,2);
            vx1=x(i,3);
            vy1=x(i,4);
            vx2=x(i,5);
            vy2=x(i,6);
            w=x(i,7);
            v1=[vx1,vy1];
            v2=[vx2,vy2];
            L1 = norm(v1);
            L2 = norm(v2);
            dx1 = 0;
            dy1 = 0;
            dvx1 = w*vx1/L1;
            dvy1 = w*vy1/L1;
            dvx2 = w*vx2/L2;
            dvy2 = w*vy2/L2;
            dw = L1+L2;
            g(i,:) = [-dx1, -dy1, -dvx1, -dvy1, -dvx2, -dvy2, -dw];
        else %center
            x1 = x(i,1);
            y1 = x(i,2);
            w = x(i,3);
            l = x(i,4);
            theta = x(i,5); 
            k = x(i,6); 
            r = x(i,7);
            g(i,:) = (-1)*[0, 0, l+((2*w)/tan(r/2)), w, 0, 0, -(w^2/2)*((csc(r/2))^2)];
        end
    end
end

function a=totalArea(x)
    a = 0;
    for i=1:size(x,1)
        w = x(i,3);
        l = x(i,4);
        r = x(i,7);
        a = a + (l*w + w^2/tan(r/2));       
    end
end

function a=totalAreaLineWidth(x)
    a=0;
    for i=1:size(x,1)
        b=x(i,:);
        x1 = b(1);
        y1 = b(2);
        x2 = b(3);
        y2 = b(4);
        w = b(5);
        k = b(6);
        h = b(7);
        l = sqrt((x2-x1)^2 + (y2-y1)^2); %Length between the two given points
        beta = atan(h/k); %Angle of p1 corner point
        alpha = atan(h/(1-k)); %Angle of p2 corner point
        r = pi - beta - alpha; %Kink angle
        L = l*(sqrt(h^2+k^2)+sqrt(h^2+(1-k)^2));
        %Area of unkinked rectangle plus extra area of kink area
        % Have to adjust length 
        a = a + w*(L-w/tan(r/2)) + w^2/tan(r/2);
    end
end

function a=totalAreaLowerLineWidth(x)
    a=0;
    for i=1:size(x,1)
        b=x(i,:);
        x1 = b(1);
        y1 = b(2);
        x2 = b(3);
        y2 = b(4);
        w = b(5);
        k = b(6);
        h = b(7);
        l = sqrt((x2-x1)^2 + (y2-y1)^2); %Length between the two given points
        beta = atan(h/k); %Angle of p1 corner point
        alpha = atan(h/(1-k)); %Angle of p2 corner point
        r = pi - beta - alpha; %Kink angle
        L = l*(sqrt(h^2+k^2)+sqrt(h^2+(1-k)^2));
        %Area of unkinked rectangle plus extra area of kink area
        % Have to adjust length 
        a = a + w*L + w^2/tan(r/2);
    end
end

function a=totalAreaVector(x)
    a=0;
    for i=1:size(x,1)
        x1=x(i,1);
        y1=x(i,2);
        vx1=x(i,3);
        vy1=x(i,4);
        vx2=x(i,5);
        vy2=x(i,6);
        w=x(i,7);
        v1=[vx1,vy1];
        v2=[vx2,vy2];
        cross = @(v1,v2) v1(1)*v2(2)-v1(2)*v2(1);
        % Add area for kink area
        n1 = [-vy1,vx1]/norm(v1);
        n2 = [-vy2,vx2]/norm(v2);
        midK= mean([n1;n2]);
        nK = midK/norm(midK);
        K = w/dot(n2,nK);
        ka = 0.5*w*(cross(K*nK,n1)+cross(n2,K*nK));
        a = a + norm(v1)*w + norm(v2)*w + ka;
    end
end

function a=totalAreaMidVector(x)
    a=0;
    for i=1:size(x,1)
        x1=x(i,1);
        y1=x(i,2);
        vx1=x(i,3);
        vy1=x(i,4);
        vx2=x(i,5);
        vy2=x(i,6);
        w=x(i,7);
        v1=[vx1,vy1];
        v2=[vx2,vy2];
        a = a +w*(norm(v1)+norm(v2));
    end
end


