function [xout, fval, exitflag, iters]=optimize(SITE, NUM_BUILDINGS, INITIAL_SEED, SHOULD_TRANSFORM_VARIABLES, initialStrat)
        
    CONVEX_HULL_SITE = getConvexHull(SITE);
    if isequal(SITE,CONVEX_HULL_SITE)
        CONVEX_STATIC_OBJECTS = {};
    else
        staticObjects = getStaticObjects(SITE);
        CONVEX_STATIC_OBJECTS = {};
        k = 1;
        for i=1:length(staticObjects)
            if ~isempty(staticObjects{i})
                convObjs = convexDecomposition(staticObjects{i});
                for j=1:length(convObjs)
                    CONVEX_STATIC_OBJECTS{k} = convObjs{j};
                    k = k + 1;
                end
            end
        end
    end
    
    %plotSite(SITE,'b');
    %plotPolygons(CONVEX_STATIC_OBJECTS);
    %return
    
    disp('GENERATING INITIAL VALUES...');   
    x0 = generateInitialValues(initialStrat, INITIAL_SEED, SITE, NUM_BUILDINGS);
    if SHOULD_TRANSFORM_VARIABLES == Configuration.LINEWIDTH || ...
            SHOULD_TRANSFORM_VARIABLES == Configuration.LOWERLINEWIDTH
        rect0 = transformVariables(x0(:,1:5), 'to');
        rect0(:,6) = 0.5;
        rect0(:,7) = 0.0;
        x0 = rect0;
    elseif SHOULD_TRANSFORM_VARIABLES == Configuration.CENTER && initialStrat ~= Configuration.INIT_ANGLEDOPENZONES
        x0(:,6) = x0(:,4)/2;
        x0(:,7) = pi;
    elseif SHOULD_TRANSFORM_VARIABLES == Configuration.VECTOR
        rect0 = transformVariables(x0(:,1:5), 'to');
        %rect0(:,6) = 0.5;
        %rect0(:,7) = 0.0;
        x0 = [];
        for i=1:size(rect0,1)
            p1=rect0(i,1:2);
            p3=rect0(i,3:4);
            p2=mean([p1;p3]);
            vec1=p2-p1;
            vec2=p3-p2;
            x0(i,:) = [p1,vec1,vec2,rect0(i,5)];
        end
    elseif SHOULD_TRANSFORM_VARIABLES == Configuration.MIDVECTOR
        rect0 = transformVariables(x0(:,1:5), 'to');
        x0 = [];
        for i = 1:size(rect0,1)
            c1=rect0(i,1:2);
            c3=rect0(i,3:4);
            vec=c3-c1;
            w=rect0(i,5);
            n1 = [-vec(2), vec(1)]/norm(vec);
            p1 = c1 + n1*(w/2);
            p2 = p1 + 0.5*vec;
            p3 = p1 + vec;
            vec1 = p2-p1;
            vec2 = p3-p2;
            x0(i,:) = [p1,vec1,vec2,rect0(i,5)];
        end
    end

    %x0t = transformVariables(x0, 'to');
    disp('...DONE');
    
    % To save value from each iteration
    history = [];
    
    % Variable bounds
    if SHOULD_TRANSFORM_VARIABLES == Configuration.LINEWIDTH
        lbKink = Configuration.MIN_KINK/Configuration.MIN_LENGTH;
        ubKink = 1-Configuration.MIN_KINK/Configuration.MIN_LENGTH;
        LOWER_BOUNDS = repmat([min(SITE)- Configuration.MIN_LENGTH, min(SITE)- Configuration.MIN_LENGTH, Configuration.MIN_WIDTH, lbKink, -0.5], NUM_BUILDINGS, 1);
        UPPER_BOUNDS = repmat([max(SITE)+ Configuration.MAX_LENGTH, max(SITE)+ Configuration.MAX_LENGTH, Configuration.MAX_WIDTH, ubKink, 0.5], NUM_BUILDINGS, 1);
    elseif SHOULD_TRANSFORM_VARIABLES == Configuration.LOWERLINEWIDTH
        lbKink = Configuration.MIN_KINK/Configuration.MIN_LENGTH;
        ubKink = 1-Configuration.MIN_KINK/Configuration.MIN_LENGTH;
        LOWER_BOUNDS = repmat([min(SITE)- Configuration.MIN_LENGTH, min(SITE)- Configuration.MIN_LENGTH, Configuration.MIN_WIDTH, lbKink, 0.0], NUM_BUILDINGS, 1);
        UPPER_BOUNDS = repmat([max(SITE)+ Configuration.MAX_LENGTH, max(SITE)+ Configuration.MAX_LENGTH, Configuration.MAX_WIDTH, ubKink, 0.5], NUM_BUILDINGS, 1);
    elseif SHOULD_TRANSFORM_VARIABLES == Configuration.VECTOR || SHOULD_TRANSFORM_VARIABLES == Configuration.UPDOWNVECTOR || SHOULD_TRANSFORM_VARIABLES == Configuration.MIDVECTOR
        LOWER_BOUNDS = repmat([min(SITE)- Configuration.MIN_LENGTH, -Configuration.MAX_LENGTH,-Configuration.MAX_LENGTH,-Configuration.MAX_LENGTH,-Configuration.MAX_LENGTH, Configuration.MIN_WIDTH], NUM_BUILDINGS, 1);
        UPPER_BOUNDS = repmat([max(SITE)+ Configuration.MAX_LENGTH, Configuration.MAX_LENGTH,Configuration.MAX_LENGTH,Configuration.MAX_LENGTH,Configuration.MAX_LENGTH, Configuration.MAX_WIDTH], NUM_BUILDINGS, 1);
    else
        LOWER_BOUNDS = repmat([min(SITE) - Configuration.MIN_LENGTH, Configuration.MIN_WIDTH, Configuration.MIN_LENGTH, -4*pi, 0 , pi/2], NUM_BUILDINGS, 1);
        UPPER_BOUNDS = repmat([max(SITE) + Configuration.MIN_LENGTH, Configuration.MAX_WIDTH, Configuration.MAX_LENGTH, 4*pi, Configuration.MAX_LENGTH, pi], NUM_BUILDINGS, 1);
    end
    
    % Linear site inequalities
    SITE_INEQUALITIES = createLinearShortestDistance(CONVEX_HULL_SITE);

    % 'StepTolerance', 1e-20
    % TO USE OLD VARIABLE FORMULATION USE TRUE FOR SPECIFYOBJECTIVEFUNCTION
    options = optimoptions(@fmincon, 'Display', 'iter-detailed', 'Algorithm', 'sqp',...     
        'SpecifyObjectiveGradient',false,'CheckGradients',false,'ScaleProblem', true,... %, 'OutputFcn',@outfun,... %,...
        'MaxFunctionEvaluations', 100000, 'MaxIterations', 400);%, 'StepTolerance', 1e-30, 'MaxFunctionEvaluations', 100000);
    %options = optimoptions(@fmincon, 'Display', 'iter', 'Algorithm', 'interior-point');
     
    disp('OPTIMIZING...');
    if SHOULD_TRANSFORM_VARIABLES == Configuration.LINEWIDTH || ...
            SHOULD_TRANSFORM_VARIABLES == Configuration.LOWERLINEWIDTH || ...
            SHOULD_TRANSFORM_VARIABLES == Configuration.UPDOWNVECTOR || ...
            SHOULD_TRANSFORM_VARIABLES == Configuration.VECTOR || ...
            SHOULD_TRANSFORM_VARIABLES == Configuration.MIDVECTOR
        [A, b]  = getInequalityMatrix(SITE_INEQUALITIES, NUM_BUILDINGS);
        [x, fval, exitflag,output] = ...
        fmincon(@(x)objfun(x, SHOULD_TRANSFORM_VARIABLES), x0, [], [], [], [], LOWER_BOUNDS, UPPER_BOUNDS, @(x)confun(x, CONVEX_STATIC_OBJECTS, SITE_INEQUALITIES, SHOULD_TRANSFORM_VARIABLES), options);
        %fmincon(@(x)objfun(x, SHOULD_TRANSFORM_VARIABLES), x0, A, b, [], [], LOWER_BOUNDS, UPPER_BOUNDS, @(x)confun(x, CONVEX_STATIC_OBJECTS, SITE_INEQUALITIES, SHOULD_TRANSFORM_VARIABLES), options);
    else
        [x, fval, exitflag,output] = ...
        fmincon(@(x)objfun(x, SHOULD_TRANSFORM_VARIABLES), x0, [], [], [], [], LOWER_BOUNDS, UPPER_BOUNDS, @(x)confun(x, CONVEX_STATIC_OBJECTS, SITE_INEQUALITIES, SHOULD_TRANSFORM_VARIABLES), options);
    end
    disp('...DONE');

    function stop=outfun(x, optimValues, state)
        if SHOULD_TRANSFORM_VARIABLES == Configuration.LINEWIDTH
            xout = transformVariables(x, 'from');
        else
            xout = x;
        end
        stop = false;
        if isequal(state,'iter')
            history = [history; xout];
        end
    end
    
    %if SHOULD_TRANSFORM_VARIABLES == Configuration.LINEWIDTH
    %    xout = transformVariables(x, 'from');
    %else
    %    xout = x;   
    %end
    %plotSolution(x, SITE, SHOULD_TRANSFORM_VARIABLES);
    %animateSolution(history);
    iters=output.iterations;
    xout=x;
    end

function [A,b] = getInequalityMatrix(SITE_INEQUALITIES, NUM_BUILDINGS)
    numInequalities = size(SITE_INEQUALITIES,1);
    % Endpoints on LINE must lie within site
    A = zeros(2*numInequalities*NUM_BUILDINGS, 7*NUM_BUILDINGS);
    b = repmat(SITE_INEQUALITIES(:,3), 2*NUM_BUILDINGS, 1);

    for i=1:NUM_BUILDINGS
        j = 2*(i-1);
        firstR = [j*numInequalities + 1;(j+1)*numInequalities];
        secondR = [(j+1)*numInequalities + 1; (j+2)*numInequalities];
        
        firstC = [7*(i-1) + 1;7*(i-1) + 2];
        secondC = [7*(i-1) + 3;7*(i-1) + 4];
        
        A(firstR(1):firstR(2), firstC(1):firstC(2)) = -SITE_INEQUALITIES(:,1:2);
        A(secondR(1):secondR(2), secondC(1):secondC(2)) = -SITE_INEQUALITIES(:,1:2);

    end
    
    for j=1:size(A,1)
        r = A(j,:);
        mat = vec2mat(r, 7);
        for e=1:numel(mat)
            r(e) = mat(e);
        end
        A(j,:) = r;
    end
    
end
   
        