function [xout,fout,exitflag]= optimizeHeights(X,ZONES,ZONES_HEIGHTS,MEAN_HEIGHT, CONFIG)
    numVars=size(X,1);
    LOWER_BOUNDS=ones(numVars,1)*Configuration.MIN_HEIGHT;
    UPPER_BOUNDS=buildingsToZoneHeight(X,ZONES,ZONES_HEIGHTS);
    h0=LOWER_BOUNDS;
    
    options = optimoptions(@fmincon, 'Display', 'iter-detailed', 'Algorithm', 'sqp',...     
        'SpecifyObjectiveGradient',false,'CheckGradients',false,'ScaleProblem', true,...
        'MaxFunctionEvaluations', 100000, 'MaxIterations', 400);
    
    % Optimize heights of buildings
    [h,~,exitflag] = ...
        fmincon(@(h)objfun3d([X,h], CONFIG), h0, [], [], [], [], LOWER_BOUNDS, UPPER_BOUNDS, @(h)heightcons(h,X,MEAN_HEIGHT,CONFIG), options);
    
    xout=[X,h];
    fout=objfun3d(xout,CONFIG);
end

function [cons,ceq]=heightcons(h,X,MEAN_HEIGHT, TRANSFORM)
    cons=[];
    ceq=[];
    footprintCornersVars = variablesToCorners(X,TRANSFORM);
    numVars=size(X,1);
    % No internal overlap constraint
    k = 1;
    for i=1:numVars
        iCorners=footprintCornersVars((6*(i-1) + 1):(6*(i-1) + 6),:);
        iHeight=h(i);
        for j=1:numVars
            if i~=j
                jCorners = footprintCornersVars((6*(j-1) + 1):(6*(j-1) + 6),:);
                jHeight=h(j);
                % Length
                % Length buffer zones (both ways)
                ijLengthCons = lengthOpenZoneOverlap(iCorners,jCorners,jHeight)-sqrt(eps);
                cons(k:(k+length(ijLengthCons)-1),1)=ijLengthCons;
                k = k + length(ijLengthCons);

                jiLengthCons = lengthOpenZoneOverlap(jCorners,iCorners,iHeight)-sqrt(eps);
                cons(k:(k+length(jiLengthCons)-1),1)=jiLengthCons;
                k = k + length(jiLengthCons);
            end
        end
    end
    % Average height
    cons(k)=sum(h)-numVars*MEAN_HEIGHT;
    
end


function corners=variablesToCorners(B, TRANSFORM)
    numVars = size(B, 1);
    C=zeros(6*numVars, 2);
    for i=1:numVars
        bVar = B(i,:);
        bCorners = altCornerPoints(bVar, TRANSFORM);
        n1 = 6*(i-1) + 1;
        n2 = 6*(i-1) + 6;
        C(n1:n2,:) = bCorners;
    end
    corners=C;
end


function btzh=buildingsToZoneHeight(x,ZONES,ZONES_HEIGHTS)
    btzh=zeros(size(x,1),1);
    for i=1:size(x,1)
        bCorners=altCornerPoints(x(i,:),Configuration.CENTER);
        zminh=Inf;
        for j=1:length(ZONES)
            if sum(any(siteOverlapNonConvex(bCorners,ZONES{j}) < 0)) >= 1
               % Building overlaps with zone
               if ZONES_HEIGHTS(j) < zminh
                  zminh=ZONES_HEIGHTS(j);
               end
            end
        end
        if zminh == Inf
            error('Building does not overlap with any zones.'); 
        end
        btzh(i)=zminh;
    end
end