function bestInitialSolution = perpendicularTest(seed, SITE, NUM_BUILDINGS)
    %siteCenterPoint = mean(SITE);
    plotSite(SITE);
    validEdges = validSiteEdges(SITE);
    numValidEdges = size(validEdges,1);
    bestInitialArea = 0;
    bestInitialSolution = zeros(NUM_BUILDINGS, 5);
    for e=1:numValidEdges
        fromIndex = validEdges(e,1);
        toIndex = validEdges(e,2);
        edge = [SITE(fromIndex,:); SITE(toIndex,:)];
        edgeVector = edge(2,:) - edge(1,:);
        length = sqrt(edgeVector(1)^2 + edgeVector(2)^2);
        edgeNormal = -1*[-edgeVector(2),edgeVector(1)] / length;
       
        % Find max directional diameter to calculate number of rows of
        % buildings
        maxNormalDistance = 0;
        for i=1:size(SITE,1)
           if i~=fromIndex && i~=toIndex
               r = SITE(fromIndex,:) - SITE(i,:);
               distance = abs(dot(edgeNormal, r));
               maxNormalDistance = max(maxNormalDistance, distance);
           end
        end
        maxNumRows = floor(maxNormalDistance / (Configuration.MIN_WIDTH + (Configuration.LENGTH_OPEN_ZONE)/2));
        minNumRows = floor(maxNormalDistance / (Configuration.MAX_WIDTH + Configuration.LENGTH_OPEN_ZONE));
    
        x0 = zeros(NUM_BUILDINGS, 5);
        width = Configuration.MIN_WIDTH;
        %if NUM_BUILDINGS < minNumRows
        %    width = Configuration.MAX_WIDTH
        buildingsPerRow = NUM_BUILDINGS / maxNumRows
        %buildingsPerRow = (maxNormalDistance / (NUM_BUILDINGS*Configuration.MIN_WIDTH + (NUM_BUILDINGS-1)*Configuration.LENGTH_OPEN_ZONE));
        [lowerSegment, fullUpperSegment, upperSegment] = getBuildingRow(edge, SITE, fromIndex, toIndex, width + Configuration.LENGTH_OPEN_ZONE);
        %line(lowerSegment(:,1), lowerSegment(:,2))
        remainingLowerSegment = lowerSegment;
        remainingUpperSegment = upperSegment;
        segmentWidth = sqrt((lowerSegment(2,1)-lowerSegment(1,1))^2 + (lowerSegment(2,2)-lowerSegment(1,2))^2);
        buildingsThisRow = floor(segmentWidth / (Configuration.MIN_LENGTH + Configuration.WIDTH_OPEN_ZONE/2));
        estimatedRowsLeft = maxNumRows;
        for b=1:NUM_BUILDINGS
            if buildingsThisRow <= 0 || segmentWidth < Configuration.MIN_LENGTH
                [cLowerSegment, cFullUpperSegment, cUpperSegment] = getBuildingRow(fullUpperSegment, SITE, fromIndex, toIndex, width + Configuration.LENGTH_OPEN_ZONE);
                if isequal(cFullUpperSegment, 0) && isequal(cUpperSegment, 0)
                    % No more room for buildings with open zone, possibly
                    % final row
                    [cLowerSegment, cFullUpperSegment, cUpperSegment] = getBuildingRow(fullUpperSegment, SITE, fromIndex, toIndex, width);
                    if isequal(cFullUpperSegment, 0) && isequal(cUpperSegment, 0)
                        break
                    else
                       lowerSegment = cLowerSegment;
                       fullUpperSegment = cFullUpperSegment;
                       upperSegment = cUpperSegment;
                   end
                else
                   lowerSegment = cLowerSegment;
                   fullUpperSegment = cFullUpperSegment;
                   upperSegment = cUpperSegment;
                end
                
                
                %line(lowerSegment(:,1), lowerSegment(:,2))
                remainingLowerSegment = lowerSegment;
                remainingUpperSegment = upperSegment;
                segmentWidth = sqrt((lowerSegment(2,1)-lowerSegment(1,1))^2 + (lowerSegment(2,2)-lowerSegment(1,2))^2);
                buildingsThisRow = floor(segmentWidth / (Configuration.MIN_LENGTH + Configuration.WIDTH_OPEN_ZONE/2));
                estimatedRowsLeft = estimatedRowsLeft - 1;
                buildingsPerRow = (NUM_BUILDINGS - b + 1)/estimatedRowsLeft
                if buildingsThisRow > buildingsPerRow && buildingsThisRow > 1
                    % Do not fill too many buildings on each row
                    %buildingsThisRow = buildingsThisRow - 1;
                end
            end
            if buildingsPerRow > 1
                upperVector = remainingUpperSegment(1,:) - remainingUpperSegment(2,:);
                newTopRight = remainingUpperSegment(2,:) + (1/buildingsThisRow)*upperVector;
                lowerVector = remainingLowerSegment(1,:) - remainingLowerSegment(2,:);
                newBottomRight = remainingLowerSegment(2,:) + (1/buildingsThisRow)*lowerVector;
            else
                % Max building size on this row
                upperVector = remainingUpperSegment(1,:) - remainingUpperSegment(2,:);
                newTopRight = remainingUpperSegment(2,:) + upperVector;
                lowerVector = remainingLowerSegment(1,:) - remainingLowerSegment(2,:);
                newBottomRight = remainingLowerSegment(2,:) + lowerVector;
            end
            %upperMean = mean([remainingUpperSegment(2,:);newTopRight]);
            %lowerMean = mean([remainingLowerSegment(2,:);newBottomRight]);
            %line(remainingLowerSegment(:,1), remainingLowerSegment(:,2))
            %line([remainingLowerSegment(2,1), newBottomRight(1)], [remainingLowerSegment(2,2), newBottomRight(2)])
            lineNormalVariables = [remainingLowerSegment(2,1), remainingLowerSegment(2,2), newBottomRight(1), newBottomRight(2), width];
            variables = transformVariables(lineNormalVariables, 'from');
            %plotBuildings(variables);
            x0(b,:) = variables;
            remainingUpperSegment = [remainingUpperSegment(1,:); newTopRight];
            remainingLowerSegment = [remainingLowerSegment(1,:); newBottomRight];
            segmentWidth = sqrt((remainingLowerSegment(2,1)-remainingLowerSegment(1,1))^2 + (remainingLowerSegment(2,2)-remainingLowerSegment(1,2))^2);
            buildingsThisRow = buildingsThisRow - 1;
        end
        % Save solution if better than previos;
        if ~any(x0(:,3) <= 0)
            % If any of the widths are zero, not all buildings were able to
            % be placed
            if sum(prod(x0(:,3:4),2), 1) > bestInitialArea
                bestInitialSolution = x0;
            end
        end
    end
    %plotSite(SITE);
    %plotBuildings(bestInitialSolution);
end

function ps = validSiteEdges(SITE)
    [nSiteCornerPoints,~] = size(SITE);
    ps = zeros(2,2);
    k = 1;
    for i=1:nSiteCornerPoints
        j = mod(i, nSiteCornerPoints) + 1;
        c=SITE(i,:);
        n=SITE(j,:);
        if sqrt((n(1)-c(1))^2+(n(2)-c(2))^2) > Configuration.MIN_LENGTH
            ps(k,:) = [i, j];
            k = k + 1;
        end
    end
end

function [lowerSegment, fullUpperSegment, upperSegment]=getBuildingRow(edge, SITE, fromIndex, toIndex, width)
        nextLine = getNextRowLine(edge, width);
        nextLineSegment = getNextRowSegment(nextLine, SITE, fromIndex);
        %line(nextLineSegment(:,1), nextLineSegment(:,2));
        if isequal(nextLineSegment,0)
            lowerSegment = edge;
            fullUpperSegment = 0;
            upperSegment = 0;
            return
        end
        edgeVector = edge(2,:) - edge(1,:);
        edgeNormal = -1*[-edgeVector(2),edgeVector(1)];
        topLeft = nextLine(2,:);
        bottomLeft = edge(2,:);
        if cross2D(edgeNormal, nextLineSegment(2,:)-edge(2,:)) < 0
            topLeft = nextLineSegment(2,:);
            % Also move bottom point the same distance
            bottomLeft = edge(2,:) + (topLeft - nextLine(2,:)); 
        end
        topRight = nextLine(1,:);
        bottomRight = edge(1,:);
        if cross2D(edgeNormal, nextLineSegment(1,:)-edge(1,:)) > 0
            topRight = nextLineSegment(1,:);
            % Also move bottom point the same distance
            bottomRight = edge(1,:) + (topRight - nextLine(1,:)); 
        end
        lowerSegment=[bottomRight; bottomLeft];
        fullUpperSegment = nextLineSegment;
        upperSegment=[topRight; topLeft];
        %line(lowerSegment(:,1), lowerSegment(:,2));
end

function seg=getNextRowSegment(nextLine, SITE, baseFrom)
    numOtherEdges = size(SITE,1) - 1;
    numCorners = numOtherEdges + 1;
    leftPoint = 0;
    rightPoint = 0;
    for i=1:numOtherEdges
        fromNext = mod(baseFrom + i - 1, numCorners) + 1;
        toNext = mod(baseFrom + (i+1) - 1, numCorners) + 1;
        ls1 = [SITE(fromNext,:) ; SITE(toNext,:)];
        t=lineSegmentsIntersectionMultiple(ls1, nextLine);
        %line([ls1(1,1), ls1(2,1)], [ls1(1,2), ls1(2,2)])
        if 0 <= t && t < 1
            if leftPoint == 0
                leftPoint = SITE(fromNext,:) + t*(SITE(toNext,:) - SITE(fromNext,:));
            elseif rightPoint == 0
                rightPoint = SITE(fromNext,:) + t*(SITE(toNext,:) - SITE(fromNext,:));
            else
                error('Found multiple intersection points when slicing site for initial values (perpendicular)')
            end
        end
    end
    if isequal(leftPoint,0) && isequal(rightPoint,0)
        seg = 0;
    else
        seg = [rightPoint; leftPoint];
    end
end

function t=lineSegmentsIntersectionMultiple(ls1,ls2)
   p = ls1(1,:);
   r = ls1(2,:)-ls1(1,:);
   
   q = ls2(1,:);
   s = ls2(2,:) - ls2(1,:);
   
   t = cross2D(q-p,s) / cross2D(r, s);
end

function c=cross2D(v1,v2)
    c = v1(1)*v2(2) - v1(2)*v2(1);
end

function nextLine=getNextRowLine(prevLine, w)
    prevVector = prevLine(2,:)-prevLine(1,:);
    normal = -1*[-prevVector(2), prevVector(1)] / sqrt(prevVector(1)^2+prevVector(2)^2);
    nextLine = prevLine + w*normal;
end