function plotSolution(x, SITE, TRANSFORM)
    footPrintCorners = zeros(6*size(x,1), 2);   
    openZonesWidth = zeros(6*size(x,1), 2);
    openZonesLength = zeros(6*size(x,1), 2);
    for i=1:size(x,1)
       from = 6*(i-1) + 1;
       to = 6*i;
       footPrintCorners(from:to,:) = altCornerPoints(x(i,:),TRANSFORM);
       openZonesWidth(from:to,:) = altCornerPoints(x(i,:),TRANSFORM, true, false);
       %openZonesLength(from:to,:) = kinkedAltVariables(x(i,:), false, true);

    end
    plotProblem(footPrintCorners, SITE, openZonesWidth, openZonesLength);
end

function plotProblem(cornerShapes, SITE, varargin)
    if length(varargin)~=0 && length(varargin)~= 2
        error('Wrong number of varargs in plotProblem')
    end
    
    % Kink point circle radius
    %rad = Configuration.LENGTH_OPEN_ZONE;
    
    % Plot open zones first so they appear below the building footprints
    openZonesWidth = varargin{1};
    %openZonesLength = varargin{2};
    numOpenZones = floor(size(openZonesWidth,1)/6);
    for r=1:numOpenZones
        from = 6*(r-1) + 1;
        to = 6*r;
        wR = openZonesWidth(from:to,:);
        % = openZonesLength(from:to,:);
        patch('Faces', 1:length(wR), 'Vertices', wR,'EdgeColor','none', 'FaceColor', [0.8 0.8 0.8], 'FaceAlpha', 0.5);
        rect = cornerShapes(from:to,:);
        % Length open zones for rect
        lengthOpenZones=getLongOpenZones(rect);
        [tri1,tri2]=getTriangleZones(lengthOpenZones,rect(2,:),rect(5,:));
        for l=1:4
            oz=lengthOpenZones(4*(l-1)+1:4*(l-1)+4,:);
            patch('Faces', 1:length(oz), 'Vertices', oz,'EdgeColor','none', 'FaceColor', [0.8 0.8 0.8], 'FaceAlpha', 0.5);
        end
        
        % Triangles
        patch('Faces', 1:length(tri1), 'Vertices', tri1,'EdgeColor','none', 'FaceColor', [0.8 0.8 0.8], 'FaceAlpha', 0.3);
        patch('Faces', 1:length(tri2), 'Vertices', tri2,'EdgeColor','none', 'FaceColor', [0.8 0.8 0.8], 'FaceAlpha', 0.3);
        
        % Kink point circles
        %rectangle('Position',[rect(2,1)-rad,rect(2,2)-rad,2*rad,2*rad], 'Curvature', [1,1],'EdgeColor','none', 'FaceColor', [0.8 0.8 0.8 0.5]);
        %rectangle('Position',[rect(5,1)-rad,rect(5,2)-rad,2*rad,2*rad], 'Curvature', [1,1],'EdgeColor','none','FaceColor', [0.8 0.8 0.8 0.5]);
        %patch('Faces', 1:length(wL),'Vertices', wL,'EdgeColor','none', 'FaceColor', [0.8 0.8 0.8], 'FaceAlpha', 0.5);

    end
    
    patch('Faces', 1:length(SITE), 'Vertices',SITE,'EdgeColor','black','FaceColor','none','LineWidth',1);
    colors =  ['r'; 'g'; 'b'; 'y'; 'm'; 'c'; 'r'; 'g'; 'b'; 'y'; 'm'; 'c'; 'r'; 'g'; 'b'; 'y'; 'm'; 'c'; 'r'; 'g'; 'b'; 'y'; 'm'; 'c'];
    numRects = floor(size(cornerShapes,1)/6);
    for r=1:numRects
        from = 6*(r-1) + 1;
        to = 6*r;
        rect = cornerShapes(from:to,:);
        patch(transpose(rect(:,1)), transpose(rect(:,2)), colors(r));        
        
    end
    axis equal;
    alpha(0.7);
end