function x0=sameOrientationRandomTest(seed, SITE, NUM_BUILDINGS)
    % First, select a random side and place a building on it, 
    rng(seed);
    
    % All initial solutions
    x0 = zeros(NUM_BUILDINGS,5);
    validEdges = validSiteEdges(SITE);
    randomRow = randi([1,size(validEdges,1)]);
    fromIndex = validEdges(randomRow,1);
    toIndex = validEdges(randomRow,2);
    baseEdge = [SITE(fromIndex,:); SITE(toIndex,:)];
    edgeVector = baseEdge(2,:) - baseEdge(1,:);
    edgeLength = sqrt(edgeVector(1)^2 + edgeVector(2)^2);
    randomLength = rand()*(min(edgeLength,Configuration.MAX_LENGTH) - Configuration.MIN_LENGTH);
    randomLength = (randomLength + Configuration.MIN_LENGTH);
    %t = rand()*(edgeLength-Configuration.MIN_LENGTH);
    randomStart = rand()*(edgeLength - randomLength);
    fromCorner = baseEdge(1,:) + randomStart*edgeVector/edgeLength;
    toCorner = fromCorner + randomLength*edgeVector/edgeLength;
    
    edges = zeros(2,2,1);
    % Number of available edges
    e = 0;
    if pointDistance(baseEdge(1,:), fromCorner) > Configuration.MIN_LENGTH
        e = e + 1; 
        edges(:,:,e) = [baseEdge(1,:); fromCorner]; 
    end
    if pointDistance(toCorner, baseEdge(2,:)) > Configuration.MIN_LENGTH
        e = e + 1; 
        edges(:,:,e) = [toCorner; baseEdge(2,:)]; 
    end
    
    lineNormalVariables = [toCorner(1), toCorner(2), fromCorner(1), fromCorner(2), Configuration.MIN_WIDTH];
    xn = transformVariables(lineNormalVariables, 'from');
    x0(1,:) = xn;
    % Would like to place buildins perpendicular to other buildings. 
    
    % The next edge that all buildings on this 'row' must lie on
    % Used to move buildings inward on the site
    width = Configuration.MIN_WIDTH + Configuration.LENGTH_OPEN_ZONE;
    upperEdge = baseEdge;
    %currentRowLine = 1;
    buildingsLeft = NUM_BUILDINGS - 1;
    while buildingsLeft > 0
        if e <= 0
            % Get next Upper Edge
            [~, cFullUpperSegment, cUpperSegment] = getBuildingRow(upperEdge, SITE, fromIndex, toIndex, width);
            if isequal(cFullUpperSegment, 0) && isequal(cUpperSegment, 0)
                % No more space in site
                %error('No more space on site');
                x0=[];
                return
                %break
            else
               upperEdge = cFullUpperSegment;
            end
             % No more space on this edge, move forward
            if pointDistance(upperEdge(1,:), upperEdge(2,:)) > Configuration.MIN_LENGTH
                e = e + 1; 
                edges(:,:,e) = [upperEdge(1,:); upperEdge(2,:)]; 
            end

        end
        r = randi([1,e]);
        baseEdge = edges(:,:,r);
        edgeVector = baseEdge(2,:) - baseEdge(1,:);
        edgeLength = sqrt(edgeVector(1)^2 + edgeVector(2)^2);
        randomLength = rand()*(min(edgeLength,Configuration.MAX_LENGTH) - Configuration.MIN_LENGTH);
        randomLength = (randomLength + Configuration.MIN_LENGTH);
        %t = rand()*(edgeLength-Configuration.MIN_LENGTH);
        randomStart = rand()*(edgeLength - randomLength);
        fromCorner = baseEdge(1,:) + randomStart*edgeVector/edgeLength;
        toCorner = fromCorner + randomLength*edgeVector/edgeLength;
        
        % Add building 
        lineNormalVariables = [toCorner(1), toCorner(2), fromCorner(1), fromCorner(2), Configuration.MAX_WIDTH];
        xn = transformVariables(lineNormalVariables, 'from');
        x0(NUM_BUILDINGS-buildingsLeft+1,:) = xn;
        
        % Remove edge from list of available edges
        
        edges(:,:,r:size(edges,3)-1) = edges(:,:,r+1:size(edges,3));
        edges(:,:,size(edges,3)) = [];
        e = e - 1;
        if pointDistance(baseEdge(1,:), fromCorner) > Configuration.MIN_LENGTH
            e = e + 1; 
            edges(:,:,e) = [baseEdge(1,:); fromCorner]; 
        end
        if pointDistance(toCorner, baseEdge(2,:)) > Configuration.MIN_LENGTH
            e = e + 1; 
            edges(:,:,e) = [toCorner; baseEdge(2,:)]; 
        end
        
        buildingsLeft = buildingsLeft - 1;
    end
end

function d=pointDistance(p1,p2)
    d=sqrt((p2(1) - p1(1))^2 + (p2(2) - p1(2))^2);
end

function ps = validSiteEdges(SITE)
    [nSiteCornerPoints,~] = size(SITE);
    ps = zeros(2,2);
    k = 1;
    for i=1:nSiteCornerPoints
        j = mod(i, nSiteCornerPoints) + 1;
        c=SITE(i,:);
        n=SITE(j,:);
        if sqrt((n(1)-c(1))^2+(n(2)-c(2))^2) > Configuration.MIN_LENGTH
            ps(k,:) = [i, j];
            k = k + 1;
        end
    end
end


function [lowerSegment, fullUpperSegment, upperSegment]=getBuildingRow(edge, SITE, fromIndex, toIndex, width)
        nextLine = getNextRowLine(edge, width);
        nextLineSegment = getNextRowSegment(nextLine, SITE, fromIndex);
        %line(nextLineSegment(:,1), nextLineSegment(:,2));
        if isequal(nextLineSegment,0)
            lowerSegment = edge;
            fullUpperSegment = 0;
            upperSegment = 0;
            return
        end
        edgeVector = edge(2,:) - edge(1,:);
        edgeNormal = -1*[-edgeVector(2),edgeVector(1)];
        topLeft = nextLine(2,:);
        bottomLeft = edge(2,:);
        if cross2D(edgeNormal, nextLineSegment(2,:)-edge(2,:)) < 0
            topLeft = nextLineSegment(2,:);
            % Also move bottom point the same distance
            bottomLeft = edge(2,:) + (topLeft - nextLine(2,:)); 
        end
        topRight = nextLine(1,:);
        bottomRight = edge(1,:);
        if cross2D(edgeNormal, nextLineSegment(1,:)-edge(1,:)) > 0
            topRight = nextLineSegment(1,:);
            % Also move bottom point the same distance
            bottomRight = edge(1,:) + (topRight - nextLine(1,:)); 
        end
        lowerSegment=[bottomRight; bottomLeft];
        fullUpperSegment = nextLineSegment;
        upperSegment=[topRight; topLeft];
        %line(lowerSegment(:,1), lowerSegment(:,2));
end

function seg=getNextRowSegment(nextLine, SITE, baseFrom)
    numOtherEdges = size(SITE,1) - 1;
    numCorners = numOtherEdges + 1;
    leftPoint = 0;
    rightPoint = 0;
    for i=1:numOtherEdges
        fromNext = mod(baseFrom + i - 1, numCorners) + 1;
        toNext = mod(baseFrom + (i+1) - 1, numCorners) + 1;
        ls1 = [SITE(fromNext,:) ; SITE(toNext,:)];
        t=lineSegmentsIntersectionMultiple(ls1, nextLine);
        %line([ls1(1,1), ls1(2,1)], [ls1(1,2), ls1(2,2)])
        if 0 <= t && t < 1
            if leftPoint == 0
                leftPoint = SITE(fromNext,:) + t*(SITE(toNext,:) - SITE(fromNext,:));
            elseif rightPoint == 0
                rightPoint = SITE(fromNext,:) + t*(SITE(toNext,:) - SITE(fromNext,:));
            else
                error('Found multiple intersection points when slicing site for initial values (perpendicular)')
            end
        end
    end
    if isequal(leftPoint,0) && isequal(rightPoint,0)
        seg = 0;
    else
        seg = [rightPoint; leftPoint];
    end
end

function t=lineSegmentsIntersectionMultiple(ls1,ls2)
   p = ls1(1,:);
   r = ls1(2,:)-ls1(1,:);
   
   q = ls2(1,:);
   s = ls2(2,:) - ls2(1,:);
   
   t = cross2D(q-p,s) / cross2D(r, s);
end

function c=cross2D(v1,v2)
    c = v1(1)*v2(2) - v1(2)*v2(1);
end

function nextLine=getNextRowLine(prevLine, w)
    prevVector = prevLine(2,:)-prevLine(1,:);
    normal = -1*[-prevVector(2), prevVector(1)] / sqrt(prevVector(1)^2+prevVector(2)^2);
    nextLine = prevLine + w*normal;
end