function d=siteOverlap(rectCorners, site)
    % Varargin are convex sub-polygons of a non-convex site. If the site 
    % is convex, it is the only argument
    d = -inf;
    for c=1:(size(rectCorners,1))
        corner = rectCorners(c,:);
        minD = inf;
        poly = site;
        isPointInPoly = pointInPolygon(corner, poly);
        N = size(poly,1);       
        for i=1:N
            j =  mod(i, size(poly,1)) + 1;
            distance = pointToLineSegmentDistance(corner, poly(i,:), poly(j,:));
            if isPointInPoly == 1 && minD <= 0
                minD=max(-distance, minD);
            elseif isPointInPoly == 1 && minD > 0
                minD=min(-distance, minD);
            elseif isPointInPoly == 0 && minD <= 0
                % pass
            elseif isPointInPoly == 0 && minD > 0
                minD=min(distance, minD);
            end
        end
        if minD > 0 && d > 0
            d = max(d, minD);
        elseif minD <= 0 && d <= 0       
            d = max(d, minD);
        elseif minD > 0 && d <= 0
            d = minD;
        end
    end
end

function in=pointInPolygon(point, poly)
    % Traverse the convex polygon clockwise, evaluate cross products
    % between boundary segments and point.
    for i=1:size(poly,1)
        j = mod(i, size(poly,1)) + 1;
        polyV = poly(j,:) - poly(i,:);
        pointV = point - poly(i,:);
        cross = polyV(1)*pointV(2)-polyV(2)*pointV(1);
        if (cross > 0)
            in=false;
            return
        end
    end
    in=true;
end


% https://math.stackexchange.com/questions/330269/the-distance-from-a-point-to-a-line-segment
function d=pointToLineSegmentDistance(p, s1, s2)
    pointVector = p - s1;
    segmentVector = s2 - s1;
    segmentLengthSquared = (s2(1) - s1(1))^2 + (s2(2) - s1(2))^2;
    tHat = dot(pointVector, segmentVector) / segmentLengthSquared;
    tStar = min(max(tHat, 0), 1);   
    closestPointOnSegment = s1 + tStar*(s2-s1);
    distance = sqrt( (closestPointOnSegment(1)-p(1))^2 + (closestPointOnSegment(2)-p(2))^2);
    d=distance;
end
