function [xout, fval, exitflag, history]=subOptimize(SITE, ZONES,ZONES_HEIGHTS, B_TO_Z, INIT_ZONES, B_TO_IZ,NUM_BUILDINGS,FIXED_BUILDINGS, MEAN_HEIGHT, INITIAL_SEED, SHOULD_TRANSFORM_VARIABLES)
    
    ZONES_STATIC_OBJECTS={};
    nzso=1;
    for i=1:length(ZONES)
        zso=getStaticObjects(ZONES{i});
        if ~isempty(zso)
           decomposed={};
           dn=1;
           for j=1:length(zso)
              decompedStatic=convexDecomposition(zso{j});
              for k=1:length(decompedStatic)
                  decomposed{dn}=decompedStatic{k};
                  dn=dn+1;
              end
           end
           ZONES_STATIC_OBJECTS{nzso}=decomposed;
        else
           ZONES_STATIC_OBJECTS{nzso}={};
        end
        nzso=nzso+1;
    end
    % Create cellarray such that element b contains staticobjs for building
    % b
    BUILDINGS_STATIC_OBJECTS={};
    for i=1:NUM_BUILDINGS
        BUILDINGS_STATIC_OBJECTS{i}=ZONES_STATIC_OBJECTS{B_TO_Z(i)};
    end

    disp('GENERATING INITIAL VALUES...');
    % Draw random within each zone
    initBSeed=INITIAL_SEED*10000;
    for i=1:NUM_BUILDINGS
        zone=INIT_ZONES{B_TO_IZ(i)};
        x0(i,:) = generateInitialValues(Configuration.INIT_RANDOM, initBSeed, zone, 1);
        initBSeed = initBSeed + 1;
        
    end
    if SHOULD_TRANSFORM_VARIABLES == Configuration.LINEWIDTH || ...
            SHOULD_TRANSFORM_VARIABLES == Configuration.LOWERLINEWIDTH
        rect0 = transformVariables(x0(:,1:5), 'to');
        rect0(:,6) = 0.5;
        rect0(:,7) = 0.0;
        rect0(:,8) = Confgiuration.MIN_HEIGHT;
        x0 = rect0;
    elseif SHOULD_TRANSFORM_VARIABLES == Configuration.CENTER
        x0(:,6) = x0(:,4)/2;
        x0(:,7) = pi;
        x0(:,8) = Configuration.MIN_HEIGHT;
    elseif SHOULD_TRANSFORM_VARIABLES == Configuration.VECTOR
        rect0 = transformVariables(x0(:,1:5), 'to');
        x0 = [];
        for i=1:size(rect0,1)
            p1=rect0(i,1:2);
            p3=rect0(i,3:4);
            p2=mean([p1;p3]);
            vec1=p2-p1;
            vec2=p3-p2;
            x0(i,:) = [p1,vec1,vec2,rect0(i,5), Configuration.MIN_HEIGHT];
        end
    elseif SHOULD_TRANSFORM_VARIABLES == Configuration.MIDVECTOR
        rect0 = transformVariables(x0(:,1:5), 'to');
        x0 = [];
        for i = 1:size(rect0,1)
            c1=rect0(i,1:2);
            c3=rect0(i,3:4);
            vec=c3-c1;
            w=rect0(i,5);
            n1 = [-vec(2), vec(1)]/norm(vec);
            p1 = c1 + n1*(w/2);
            p2 = p1 + 0.5*vec;
            p3 = p1 + vec;
            vec1 = p2-p1;
            vec2 = p3-p2;
            x0(i,:) = [p1,vec1,vec2,rect0(i,5), Configuration.MIN_HEIGHT];
        end
    end
    
    %x0t = transformVariables(x0, 'to');
    disp('...DONE');

    % To save value from each iteration
    history = [];
    
    % Variable bounds
    if SHOULD_TRANSFORM_VARIABLES == Configuration.LINEWIDTH
        lbKink = Configuration.MIN_KINK/Configuration.MIN_LENGTH;
        ubKink = 1-Configuration.MIN_KINK/Configuration.MIN_LENGTH;
        LOWER_BOUNDS = repmat([min(SITE)- Configuration.MIN_LENGTH, min(SITE)- Configuration.MIN_LENGTH, Configuration.MIN_WIDTH, lbKink, -0.5], NUM_BUILDINGS, 1);
        UPPER_BOUNDS = repmat([max(SITE)+ Configuration.MAX_LENGTH, max(SITE)+ Configuration.MAX_LENGTH, Configuration.MAX_WIDTH, ubKink, 0.5], NUM_BUILDINGS, 1);
    elseif SHOULD_TRANSFORM_VARIABLES == Configuration.LOWERLINEWIDTH
        lbKink = Configuration.MIN_KINK/Configuration.MIN_LENGTH;
        ubKink = 1-Configuration.MIN_KINK/Configuration.MIN_LENGTH;
        LOWER_BOUNDS = repmat([min(SITE)- Configuration.MIN_LENGTH, min(SITE)- Configuration.MIN_LENGTH, Configuration.MIN_WIDTH, lbKink, 0.0], NUM_BUILDINGS, 1);
        UPPER_BOUNDS = repmat([max(SITE)+ Configuration.MAX_LENGTH, max(SITE)+ Configuration.MAX_LENGTH, Configuration.MAX_WIDTH, ubKink, 0.5], NUM_BUILDINGS, 1);
    elseif SHOULD_TRANSFORM_VARIABLES == Configuration.VECTOR || SHOULD_TRANSFORM_VARIABLES == Configuration.UPDOWNVECTOR || SHOULD_TRANSFORM_VARIABLES == Configuration.MIDVECTOR
        LOWER_BOUNDS = repmat([min(SITE)- Configuration.MIN_LENGTH, -Configuration.MAX_LENGTH,-Configuration.MAX_LENGTH,-Configuration.MAX_LENGTH,-Configuration.MAX_LENGTH, Configuration.MIN_WIDTH], NUM_BUILDINGS, 1);
        UPPER_BOUNDS = repmat([max(SITE)+ Configuration.MAX_LENGTH, Configuration.MAX_LENGTH,Configuration.MAX_LENGTH,Configuration.MAX_LENGTH,Configuration.MAX_LENGTH, Configuration.MAX_WIDTH], NUM_BUILDINGS, 1);
    else
        LOWER_BOUNDS = repmat([min(SITE) - Configuration.MIN_LENGTH, Configuration.MIN_WIDTH, Configuration.MIN_LENGTH, -4*pi, 0 , pi/2], NUM_BUILDINGS, 1);
        UPPER_BOUNDS = repmat([max(SITE) + Configuration.MIN_LENGTH, Configuration.MAX_WIDTH, Configuration.MAX_LENGTH, 4*pi, Configuration.MAX_LENGTH, pi], NUM_BUILDINGS, 1);
    end
    
    LOWER_BOUNDS(:,8)=Configuration.MIN_HEIGHT;

    % Upper bound on building heights
    for i=1:NUM_BUILDINGS
       UPPER_BOUNDS(i,8)=ZONES_HEIGHTS(B_TO_Z(i));
    end
    
    zonesInequalities={};
    for i=1:length(ZONES)
        convexZone=getConvexHull(ZONES{i});
        zonesInequalities{i}=createLinearShortestDistance(convexZone);
    end
    % Create containment inequalities for each zone
    %SITE_INEQUALITIES = createLinearShortestDistance(CONVEX_HULL_SITE);
    zoneIneqsForBuildings={};
     for i=1:NUM_BUILDINGS
        zoneIneqsForBuildings{i}=zonesInequalities{B_TO_Z(i)};
    end
    
    % 'StepTolerance', 1e-20
    % TO USE OLD VARIABLE FORMULATION USE TRUE FOR SPECIFYOBJECTIVEFUNCTION
    options = optimoptions(@fmincon, 'Display', 'iter-detailed', 'Algorithm', 'sqp',...     
        'SpecifyObjectiveGradient',false,'CheckGradients',false,'ScaleProblem', true,... %, 'OutputFcn',@outfun,... %,...
        'MaxFunctionEvaluations', 100000, 'MaxIterations', 400); %, 'StepTolerance', 1e-16);%, 'MaxFunctionEvaluations', 100000);
    %options = optimoptions(@fmincon, 'Display', 'iter', 'Algorithm', 'interior-point');
     
    disp('OPTIMIZING...');
    [x, fval, exitflag] = ...
    fmincon(@(x)objfun3d(x, SHOULD_TRANSFORM_VARIABLES), x0, [], [], [], [], LOWER_BOUNDS, UPPER_BOUNDS, @(x)subconfun(x, zoneIneqsForBuildings, BUILDINGS_STATIC_OBJECTS, FIXED_BUILDINGS, MEAN_HEIGHT, SHOULD_TRANSFORM_VARIABLES), options);
    disp('...DONE');

    function stop=outfun(x, optimValues, state)
        if SHOULD_TRANSFORM_VARIABLES == Configuration.LINEWIDTH
            xout = transformVariables(x, 'from');
        else
            xout = x;
        end
        stop = false;
        if isequal(state,'iter')
            history = [history; xout];
        end
    end
    
    %if SHOULD_TRANSFORM_VARIABLES == Configuration.LINEWIDTH
    %    xout = transformVariables(x, 'from');
    %else
    %    xout = x;   
    %end
    %plotSolution([x;FIXED_BUILDINGS], SITE, SHOULD_TRANSFORM_VARIABLES);
    %animateSolution(history);
    xout=x;
end

function [A,b] = getInequalityMatrix(SITE_INEQUALITIES, NUM_BUILDINGS)
    numInequalities = size(SITE_INEQUALITIES,1);
    % Endpoints on LINE must lie within site
    A = zeros(2*numInequalities*NUM_BUILDINGS, 7*NUM_BUILDINGS);
    b = repmat(SITE_INEQUALITIES(:,3), 2*NUM_BUILDINGS, 1);

    for i=1:NUM_BUILDINGS
        j = 2*(i-1);
        firstR = [j*numInequalities + 1;(j+1)*numInequalities];
        secondR = [(j+1)*numInequalities + 1; (j+2)*numInequalities];
        
        firstC = [7*(i-1) + 1;7*(i-1) + 2];
        secondC = [7*(i-1) + 3;7*(i-1) + 4];
        
        A(firstR(1):firstR(2), firstC(1):firstC(2)) = -SITE_INEQUALITIES(:,1:2);
        A(secondR(1):secondR(2), secondC(1):secondC(2)) = -SITE_INEQUALITIES(:,1:2);

    end
    
    for j=1:size(A,1)
        r = A(j,:);
        mat = vec2mat(r, 7);
        for e=1:numel(mat)
            r(e) = mat(e);
        end
        A(j,:) = r;
    end
    
end
function corners=variablesToCorners(B, TRANSFORM)
    numVars = size(B, 1);
    C=zeros(6*numVars, 2);
    for i=1:numVars
        bVar = B(i,:);
        bCorners = altCornerPoints(bVar, TRANSFORM);
        n1 = 6*(i-1) + 1;
        n2 = 6*(i-1) + 6;
        C(n1:n2,:) = bCorners;
    end
    corners=C;
end
   
        