function [c, ceq]=subconfun(x, zoneInequalities, zoneStaticObjects,fixedBuildings, meanHeight, TRANSFORM)
    B = x;
    cons = zeros(2*size(B,1)*(size(B,1)-1), 1);

    k = 1;
    numVars = size(x,1);
    numFixed=size(fixedBuildings,1);

    footprintCornersVars = variablesToCorners(B,TRANSFORM);
    footprintCornersFixed = variablesToCorners(fixedBuildings,TRANSFORM);
    % No internal overlap constraint
    for i=1:numVars
        iCorners=footprintCornersVars((6*(i-1) + 1):(6*(i-1) + 6),:);
        iHeight=B(i,8);
        iWidthExtendedCorners = altCornerPoints(B(i,:), TRANSFORM,true, false);
        %iLengthExtendedCorners = altCornerPoints(B(i,:), SHOULD_TRANSFORM_VARIABLES,false, true);
        for j=1:numVars
            if i~=j
                jCorners = footprintCornersVars((6*(j-1) + 1):(6*(j-1) + 6),:);
                jHeight=B(j,8);
                cons(k:k+3) = -1*nonConvexBuildingOverlap(iWidthExtendedCorners,jCorners);
                %cons(k + 1) = -1*nonCongavexBuildingOverlap(iLengthExtendedCorners,jCorners);               
                %k = k + 2;
                k = k + 4;
                
                % Length
                % Length buffer zones (both ways)
                ijLengthCons = lengthOpenZoneOverlap(iCorners,jCorners,jHeight);
                cons(k:(k+length(ijLengthCons)-1),1)=ijLengthCons;
                k = k + length(ijLengthCons);

                jiLengthCons = lengthOpenZoneOverlap(jCorners,iCorners,iHeight);
                cons(k:(k+length(jiLengthCons)-1),1)=jiLengthCons;
                k = k + length(jiLengthCons);
            end
        end
    end
    
    % No overlap with fixed buildings
    for i=1:numFixed
        iWidthExtendedCorners = altCornerPoints(fixedBuildings(i,:), TRANSFORM,true, false);
        iCorners = footprintCornersFixed((6*(i-1) + 1):(6*(i-1) + 6),:);
        for j=1:numVars
            jCorners = footprintCornersVars((6*(j-1) + 1):(6*(j-1) + 6),:);
            jWidthExtendedCorners = altCornerPoints(B(i,:), TRANSFORM, true, false);
            cons(k:k+3) = -1*nonConvexBuildingOverlap(iWidthExtendedCorners,jCorners);
            k = k + 4;
            cons(k:k+3) = -1*nonConvexBuildingOverlap(jWidthExtendedCorners,iCorners);
            k = k + 4;
            
            % Length buffer zones (both ways)
            ijLengthCons = lengthOpenZoneOverlap(iCorners,jCorners);
            cons(k:(k+length(ijLengthCons)-1),1)=ijLengthCons;
            k = k + length(ijLengthCons);
            
            jiLengthCons = lengthOpenZoneOverlap(jCorners,iCorners);
            cons(k:(k+length(jiLengthCons)-1),1)=jiLengthCons;
            k = k + length(jiLengthCons);
        end
    end
    
    
    % No overlap outside zones constraint (linear in corner points)
    for i=1:numVars
        iCorners=footprintCornersVars((6*(i-1) + 1):(6*(i-1) + 6),:);
        % Add for all cornerpoints
        % Fetch correct inequalities based on zone
        inequalities=zoneInequalities{i};
        for j=1:size(iCorners,1)
            for m=1:size(inequalities,1)
                cons(k) = -( inequalities(m,1:2)*transpose(iCorners(j,:)) + inequalities(m,3) );
                k = k + 1;
            end
        end
        
        % Add no-overlap with static objects
        so=zoneStaticObjects{i};
        for j=1:length(so)
           cons(k:k+1) = -1*siteOverlapNonConvex(iCorners, so{j});
           k = k + 2; 
        end
    end

    
    if TRANSFORM == Configuration.MIDVECTOR
       for i=1:numVars
            x1=B(i,1);
            y1=B(i,2);
            vx1=B(i,3);
            vy1=B(i,4);
            vx2=B(i,5);
            vy2=B(i,6);
            w=B(i,7);
            v1=[vx1,vy1];
            v2=[vx2,vy2];
            cross = @(v1,v2) v1(1)*v2(2)-v1(2)*v2(1);
            L1 = norm(v1);
            L2 = norm(v2);
            
            % Extra length
            n1 = [-vy1,vx1]/norm(v1);
            n2 = [-vy2,vx2]/norm(v2);
            midK= mean([n1;n2]);
            nK = midK/norm(midK);
            K = (w/2)/dot(n2,nK);
            E = dot(v1/norm(v1),K*nK);
            extraLength = abs(E);
            
            % Minimum kink
            cons(k)=Configuration.MIN_KINK - (L1-extraLength);
            cons(k+1)=Configuration.MIN_KINK - (L2-extraLength);
            
            % Building Length
            cons(k+2)=L1+L2 - 2*extraLength - Configuration.MAX_LENGTH;
            cons(k+3)=Configuration.MIN_LENGTH - (L1+L2 - 2*extraLength);
            % Angle
            cons(k+4)=cross([-v1(2),v1(1)]/L1,v2/L2);
            cons(k+5)=cross(v2/L2,[v1(2),-v1(1)]/L1);
            
            k = k + 6;
       end
    end
    
    
    if TRANSFORM == Configuration.CENTER
        % Constraint for minimum and maximum kink location
        for i=1:numVars
            kinkLocation = B(i,6);
            len = B(i,4);
            cons(k) = Configuration.MIN_KINK - kinkLocation;
            cons(k+1) = kinkLocation - len + Configuration.MIN_KINK;
            k = k + 2;
        end
    end
    
    % Average height
    cons(k)=sum(B(:,8))-numVars*meanHeight;
    
    
    c = cons;
    ceq = [];
end

function corners=variablesToCorners(B, TRANSFORM)
    numVars = size(B, 1);
    C=zeros(6*numVars, 2);
    for i=1:numVars
        bVar = B(i,:);
        bCorners = altCornerPoints(bVar, TRANSFORM);
        n1 = 6*(i-1) + 1;
        n2 = 6*(i-1) + 6;
        C(n1:n2,:) = bCorners;
    end
    corners=C;
end
