function r=transformVariables(in, fromOrTo)
    % Transforms FROM two-point-and-line formulatin TO center-width-length
    % if fromOrTo='from', opposite if 'true'
    if strcmp(fromOrTo, 'from')
        r=transformFrom(in);
    elseif strcmp(fromOrTo, 'to')
        r=transformTo(in);
    else
        error('Illegal argument in transformVariables, not "from" or "to"')
    end
   
end

function B=transformFrom(x)
    B = zeros(size(x,1), 5);
    for i=1:size(x,1)
        x1 = x(i,1);
        y1 = x(i,2);
        x2 = x(i,3);
        y2 = x(i,4);
        w = x(i,5);
        l = sqrt((x2 - x1)^2 + (y2 - y1)^2);
        centerL = [x1 + 0.5*(x2-x1), y1 + 0.5*(y2-y1)];
        centerLVector = [centerL(1) - x1, centerL(2) - y1];
        centerLNormalUnit = [-centerLVector(2), centerLVector(1)] / (l/2);
        center = centerL + 0.5*centerLNormalUnit*w;
        ydiff = y2-y1;
        xdiff = x2-x1;
        if ydiff >= 0
            cosTheta = xdiff/l;
        else
            cosTheta = -xdiff/l;
        end
        %rotation = mod(acos(cosTheta) + pi/2, 2*pi);
        rotation = acos(cosTheta);
        B(i,:) = [center(1), center(2), w, l, rotation];
    end
end


function X=transformTo(B)
    X = zeros(size(B,1), 5);
    for i=1:size(B,1)
        x = B(i,1);
        y = B(i,2);
        w = B(i,3);
        l = B(i,4);
        r = B(i,5);
        l_cos = 0.5*l*cos(r);
        w_sin = 0.5*w*sin(r);
        l_sin = 0.5*l*sin(r);
        w_cos = 0.5*w*cos(r);
        to = [x + l_cos + w_sin, y + l_sin - w_cos];
        from = [x - l_cos + w_sin, y - l_sin - w_cos];
        X(i,:) = [from(1), from(2), to(1), to(2), w];
    end
end